"""
    tests.pytests.conftest
    ~~~~~~~~~~~~~~~~~~~~~~
"""

import functools
import inspect
import logging
import os
import pathlib
import shutil
import ssl
import stat
import sys
import tempfile
import types

import attr
import pytest
from pytestshellutils.utils import ports
from saltfactories.utils import random_string

import salt.ext.tornado.ioloop
import salt.utils.files
import salt.utils.platform
from salt.serializers import yaml
from tests.conftest import FIPS_TESTRUN
from tests.support.helpers import Webserver, get_virtualenv_binary_path
from tests.support.pytest.helpers import TestAccount
from tests.support.runtests import RUNTIME_VARS

log = logging.getLogger(__name__)


@pytest.fixture(scope="session")
def salt_auth_account_1_factory():
    return TestAccount(username="saltdev-auth-1")


@pytest.fixture(scope="session")
def salt_auth_account_2_factory():
    return TestAccount(username="saltdev-auth-2", group_name="saltops")


@pytest.fixture(scope="session")
def salt_netapi_account_factory():
    return TestAccount(username="saltdev-netapi")


@pytest.fixture(scope="session")
def salt_eauth_account_factory():
    return TestAccount(username="saltdev-eauth")


@pytest.fixture(scope="session")
def salt_auto_account_factory():
    return TestAccount(username="saltdev-auto")


@pytest.fixture(scope="session")
def salt_minion_id():
    return random_string("minion-")


@pytest.fixture(scope="session")
def salt_sub_minion_id():
    return random_string("sub-minion-")


@pytest.fixture(scope="session")
def sdb_etcd_port():
    return ports.get_unused_localhost_port()


@pytest.fixture(scope="session")
def vault_port():
    return ports.get_unused_localhost_port()


@attr.s(slots=True, frozen=True)
class ReactorEvent:
    sls_path = attr.ib()
    tag = attr.ib()
    event_tag = attr.ib()


@pytest.fixture(scope="session")
def reactor_event(tmp_path_factory):

    reactor_tag = "salt/event/test"
    event_tag = random_string("test/reaction/")
    reactors_dir = tmp_path_factory.mktemp("reactors")
    reactor_test_contents = """
    reactor-test:
      runner.event.send:
        - args:
          - tag: {}
          - data:
              test_reaction: True
    """.format(
        event_tag
    )
    try:
        with pytest.helpers.temp_file(
            "reactor-test.sls", reactor_test_contents, reactors_dir
        ) as reactor_file_path:
            yield ReactorEvent(reactor_file_path, reactor_tag, event_tag)
    finally:
        shutil.rmtree(str(reactors_dir), ignore_errors=True)


@pytest.fixture(scope="session")
def master_id():
    master_id = random_string("master-")
    yield master_id


@pytest.fixture(scope="session")
def salt_master_factory(
    request,
    salt_factories,
    salt_minion_id,
    salt_sub_minion_id,
    base_env_state_tree_root_dir,
    base_env_pillar_tree_root_dir,
    prod_env_state_tree_root_dir,
    prod_env_pillar_tree_root_dir,
    ext_pillar_file_tree_root_dir,
    sdb_etcd_port,
    vault_port,
    reactor_event,
    master_id,
    salt_auth_account_1_factory,
    salt_auth_account_2_factory,
    salt_netapi_account_factory,
    salt_eauth_account_factory,
    salt_auto_account_factory,
):
    root_dir = salt_factories.get_root_dir_for_daemon(master_id)
    conf_dir = root_dir / "conf"
    conf_dir.mkdir(exist_ok=True)

    with salt.utils.files.fopen(os.path.join(RUNTIME_VARS.CONF_DIR, "master")) as rfh:
        config_defaults = yaml.deserialize(rfh.read())

        tests_known_hosts_file = str(root_dir / "salt_ssh_known_hosts")
        with salt.utils.files.fopen(tests_known_hosts_file, "w") as known_hosts:
            known_hosts.write("")

    config_defaults["root_dir"] = str(root_dir)
    config_defaults["known_hosts_file"] = tests_known_hosts_file
    config_defaults["syndic_master"] = "localhost"
    config_defaults["transport"] = request.config.getoption("--transport")
    config_defaults["reactor"] = [
        {reactor_event.tag: [str(reactor_event.sls_path)]},
    ]

    nodegroups = {
        "min": salt_minion_id,
        "sub_min": salt_sub_minion_id,
        "mins": "N@min or N@sub_min",
        "list_nodegroup": [salt_minion_id, salt_sub_minion_id],
        "multiline_nodegroup": [salt_minion_id, "or", salt_sub_minion_id],
        "one_minion_list": [salt_minion_id],
        "redundant_minions": "N@min or N@mins",
        "nodegroup_loop_a": "N@nodegroup_loop_b",
        "nodegroup_loop_b": "N@nodegroup_loop_a",
        "missing_minion": f"L@{salt_minion_id},ghostminion",
        "list_group": "N@multiline_nodegroup",
        "one_list_group": "N@one_minion_list",
        "list_group2": "N@list_nodegroup",
    }
    config_defaults["nodegroups"] = nodegroups
    config_defaults["sdbetcd"] = {
        "driver": "etcd",
        "etcd.host": "127.0.0.1",
        "etcd.port": sdb_etcd_port,
    }
    config_defaults["vault"] = {
        "url": f"http://127.0.0.1:{vault_port}",
        "auth": {"method": "token", "token": "testsecret", "uses": 0},
        "policies": ["testpolicy"],
    }

    # Config settings to test `event_return`
    config_defaults["returner_dirs"] = []
    config_defaults["returner_dirs"].append(
        os.path.join(RUNTIME_VARS.FILES, "returners")
    )
    config_defaults["event_return"] = "runtests_noop"
    config_overrides = {
        "pytest-master": {"log": {"level": "DEBUG"}},
        "fips_mode": FIPS_TESTRUN,
        "publish_signing_algorithm": (
            "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1"
        ),
    }
    ext_pillar = []
    if salt.utils.platform.is_windows():
        ext_pillar.append(
            {"cmd_yaml": "type {}".format(os.path.join(RUNTIME_VARS.FILES, "ext.yaml"))}
        )
    else:
        ext_pillar.append(
            {"cmd_yaml": "cat {}".format(os.path.join(RUNTIME_VARS.FILES, "ext.yaml"))}
        )
    ext_pillar.append(
        {
            "file_tree": {
                "root_dir": str(ext_pillar_file_tree_root_dir),
                "follow_dir_links": False,
                "keep_newline": True,
            }
        }
    )
    config_overrides["pillar_opts"] = True
    config_overrides["external_auth"] = {
        "pam": {
            salt_auth_account_1_factory.username: ["test.*"],
            f"{salt_auth_account_2_factory.group_name}%": [
                "@wheel",
                "@runner",
                "test.*",
            ],
            salt_netapi_account_factory.username: ["@wheel", "@runner", "test.*"],
            salt_eauth_account_factory.username: ["@wheel", "@runner", "test.*"],
        },
        "auto": {
            salt_netapi_account_factory.username: [
                "@wheel",
                "@runner",
                "test.*",
                "grains.*",
            ],
            salt_auto_account_factory.username: ["@wheel", "@runner", "test.*"],
            "*": ["@wheel", "@runner", "test.*"],
        },
    }

    # We need to copy the extension modules into the new master root_dir or
    # it will be prefixed by it
    extension_modules_path = str(root_dir / "extension_modules")
    if not os.path.exists(extension_modules_path):
        shutil.copytree(
            os.path.join(RUNTIME_VARS.FILES, "extension_modules"),
            extension_modules_path,
        )

    # Copy the autosign_file to the new  master root_dir
    autosign_file_path = str(root_dir / "autosign_file")
    shutil.copyfile(
        os.path.join(RUNTIME_VARS.FILES, "autosign_file"), autosign_file_path
    )
    # all read, only owner write
    autosign_file_permissions = (
        stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH | stat.S_IWUSR
    )
    os.chmod(autosign_file_path, autosign_file_permissions)

    config_overrides.update(
        {
            "ext_pillar": ext_pillar,
            "extension_modules": extension_modules_path,
            "file_roots": {
                "base": [
                    str(base_env_state_tree_root_dir),
                    os.path.join(RUNTIME_VARS.FILES, "file", "base"),
                ],
                # Alternate root to test __env__ choices
                "prod": [
                    str(prod_env_state_tree_root_dir),
                    os.path.join(RUNTIME_VARS.FILES, "file", "prod"),
                ],
            },
            "pillar_roots": {
                "base": [
                    str(base_env_pillar_tree_root_dir),
                    os.path.join(RUNTIME_VARS.FILES, "pillar", "base"),
                ],
                "prod": [str(prod_env_pillar_tree_root_dir)],
            },
        }
    )

    # Let's copy over the test cloud config files and directories into the running master config directory
    for entry in os.listdir(RUNTIME_VARS.CONF_DIR):
        if not entry.startswith("cloud"):
            continue
        source = os.path.join(RUNTIME_VARS.CONF_DIR, entry)
        dest = str(conf_dir / entry)
        if os.path.isdir(source):
            shutil.copytree(source, dest)
        else:
            shutil.copyfile(source, dest)

    factory = salt_factories.salt_master_daemon(
        master_id,
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
    )
    return factory


@pytest.fixture(scope="session")
def salt_minion_factory(salt_master_factory, salt_minion_id, sdb_etcd_port, vault_port):
    with salt.utils.files.fopen(os.path.join(RUNTIME_VARS.CONF_DIR, "minion")) as rfh:
        config_defaults = yaml.deserialize(rfh.read())
    config_defaults["hosts.file"] = os.path.join(RUNTIME_VARS.TMP, "hosts")
    config_defaults["aliases.file"] = os.path.join(RUNTIME_VARS.TMP, "aliases")
    config_defaults["transport"] = salt_master_factory.config["transport"]
    config_defaults["sdbetcd"] = {
        "driver": "etcd",
        "etcd.host": "127.0.0.1",
        "etcd.port": sdb_etcd_port,
    }
    config_defaults["vault"] = {
        "url": f"http://127.0.0.1:{vault_port}",
        "auth": {"method": "token", "token": "testsecret", "uses": 0},
        "policies": ["testpolicy"],
    }

    config_overrides = {
        "file_roots": salt_master_factory.config["file_roots"].copy(),
        "pillar_roots": salt_master_factory.config["pillar_roots"].copy(),
        "fips_mode": FIPS_TESTRUN,
        "encryption_algorithm": "OAEP-SHA224" if FIPS_TESTRUN else "OAEP-SHA1",
        "signing_algorithm": "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1",
    }

    virtualenv_binary = get_virtualenv_binary_path()
    if virtualenv_binary:
        config_overrides["venv_bin"] = virtualenv_binary
    factory = salt_master_factory.salt_minion_daemon(
        salt_minion_id,
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
    )
    factory.after_terminate(
        pytest.helpers.remove_stale_minion_key, salt_master_factory, factory.id
    )
    return factory


@pytest.fixture(scope="session")
def salt_sub_minion_factory(salt_master_factory, salt_sub_minion_id):
    with salt.utils.files.fopen(
        os.path.join(RUNTIME_VARS.CONF_DIR, "sub_minion")
    ) as rfh:
        config_defaults = yaml.deserialize(rfh.read())
    config_defaults["hosts.file"] = os.path.join(RUNTIME_VARS.TMP, "hosts")
    config_defaults["aliases.file"] = os.path.join(RUNTIME_VARS.TMP, "aliases")
    config_defaults["transport"] = salt_master_factory.config["transport"]

    config_overrides = {
        "file_roots": salt_master_factory.config["file_roots"].copy(),
        "pillar_roots": salt_master_factory.config["pillar_roots"].copy(),
        "fips_mode": FIPS_TESTRUN,
        "encryption_algorithm": "OAEP-SHA224" if FIPS_TESTRUN else "OAEP-SHA1",
        "signing_algorithm": "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1",
    }

    virtualenv_binary = get_virtualenv_binary_path()
    if virtualenv_binary:
        config_overrides["venv_bin"] = virtualenv_binary
    factory = salt_master_factory.salt_minion_daemon(
        salt_sub_minion_id,
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
    )
    factory.after_terminate(
        pytest.helpers.remove_stale_minion_key, salt_master_factory, factory.id
    )
    return factory


@pytest.fixture(scope="session")
def salt_proxy_factory(salt_master_factory):
    proxy_minion_id = random_string("proxytest-")

    config_overrides = {
        "file_roots": salt_master_factory.config["file_roots"].copy(),
        "pillar_roots": salt_master_factory.config["pillar_roots"].copy(),
        "fips_mode": FIPS_TESTRUN,
        "encryption_algorithm": "OAEP-SHA224" if FIPS_TESTRUN else "OAEP-SHA1",
        "signing_algorithm": "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1",
    }

    factory = salt_master_factory.salt_proxy_minion_daemon(
        proxy_minion_id,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
        start_timeout=240,
    )
    factory.before_start(pytest.helpers.remove_stale_proxy_minion_cache_file, factory)
    factory.after_terminate(
        pytest.helpers.remove_stale_minion_key, salt_master_factory, factory.id
    )
    factory.after_terminate(
        pytest.helpers.remove_stale_proxy_minion_cache_file, factory
    )
    return factory


@pytest.fixture(scope="session")
def salt_delta_proxy_factory(salt_factories, salt_master_factory):
    proxy_minion_id = random_string("delta-proxy-test-")
    root_dir = salt_factories.get_root_dir_for_daemon(proxy_minion_id)
    conf_dir = root_dir / "conf"
    conf_dir.mkdir(parents=True, exist_ok=True)

    config_defaults = {
        "root_dir": str(root_dir),
        "hosts.file": os.path.join(
            RUNTIME_VARS.TMP, "hosts"
        ),  # Do we really need this for these tests?
        "aliases.file": os.path.join(
            RUNTIME_VARS.TMP, "aliases"
        ),  # Do we really need this for these tests?
        "transport": salt_master_factory.config["transport"],
        "user": salt_master_factory.config["user"],
        "metaproxy": "deltaproxy",
        "master": "127.0.0.1",
    }
    config_overrides = {
        "fips_mode": FIPS_TESTRUN,
        "encryption_algorithm": "OAEP-SHA224" if FIPS_TESTRUN else "OAEP-SHA1",
        "signing_algorithm": "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1",
    }
    factory = salt_master_factory.salt_proxy_minion_daemon(
        proxy_minion_id,
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
        start_timeout=240,
    )

    for minion_id in [factory.id] + pytest.helpers.proxy.delta_proxy_minion_ids():
        factory.before_start(
            pytest.helpers.remove_stale_proxy_minion_cache_file, factory, minion_id
        )
        factory.after_terminate(
            pytest.helpers.remove_stale_minion_key, salt_master_factory, minion_id
        )
        factory.after_terminate(
            pytest.helpers.remove_stale_proxy_minion_cache_file, factory, minion_id
        )
    return factory


@pytest.fixture
def temp_salt_master(
    request,
    salt_factories,
):
    config_defaults = {
        "open_mode": True,
        "transport": request.config.getoption("--transport"),
    }
    config_overrides = {
        "fips_mode": FIPS_TESTRUN,
        "publish_signing_algorithm": (
            "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1"
        ),
    }
    factory = salt_factories.salt_master_daemon(
        random_string("temp-master-"),
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
    )
    return factory


@pytest.fixture
def temp_salt_minion(temp_salt_master):
    config_defaults = {
        "open_mode": True,
        "transport": temp_salt_master.config["transport"],
    }
    config_overrides = {
        "fips_mode": FIPS_TESTRUN,
        "encryption_algorithm": "OAEP-SHA224" if FIPS_TESTRUN else "OAEP-SHA1",
        "signing_algorithm": "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1",
    }
    factory = temp_salt_master.salt_minion_daemon(
        random_string("temp-minion-"),
        defaults=config_defaults,
        overrides=config_overrides,
        extra_cli_arguments_after_first_start_failure=["--log-level=info"],
    )
    factory.after_terminate(
        pytest.helpers.remove_stale_minion_key, temp_salt_master, factory.id
    )
    return factory


@pytest.fixture(scope="session")
def get_python_executable():
    """
    Return the path to the python executable.

    This is particularly important when running the test suite within a virtualenv, while trying
    to create virtualenvs on windows.
    """
    try:
        if salt.utils.platform.is_windows():
            python_binary = os.path.join(
                sys.real_prefix, os.path.basename(sys.executable)
            )
        else:
            python_binary = os.path.join(
                sys.real_prefix, "bin", os.path.basename(sys.executable)
            )
            if not os.path.exists(python_binary):
                if not python_binary[-1].isdigit():
                    versioned_python_binary = "{}{}".format(
                        python_binary, *sys.version_info
                    )
                    log.info(
                        "Python binary could not be found at %s. Trying %s",
                        python_binary,
                        versioned_python_binary,
                    )
                    if os.path.exists(versioned_python_binary):
                        python_binary = versioned_python_binary
        if not os.path.exists(python_binary):
            log.warning("Python binary could not be found at %s", python_binary)
            python_binary = None
    except AttributeError:
        # We're not running inside a virtualenv
        python_binary = sys.executable
    return python_binary


@pytest.fixture
def tmp_path_world_rw(request):
    """
    Temporary path which is world read/write for tests that run under a different account
    """
    tempdir_path = pathlib.Path(basetemp=tempfile.gettempdir()).resolve()
    path = tempdir_path / f"world-rw-{id(request.node)}"
    path.mkdir(exist_ok=True)
    path.chmod(0o777)
    try:
        yield path
    finally:
        shutil.rmtree(str(path), ignore_errors=True)


@pytest.fixture(scope="session")
def bridge_pytest_and_runtests():
    """
    We're basically overriding the same fixture defined in tests/conftest.py
    """


@pytest.fixture(scope="session")
def this_txt_file(integration_files_dir):
    contents = "test"
    with pytest.helpers.temp_file("this.txt", contents, integration_files_dir) as path:
        sha256sum = salt.utils.hashutils.get_hash(str(path), form="sha256")
        with pytest.helpers.temp_file(
            "this.txt.sha256", sha256sum, integration_files_dir
        ):
            yield types.SimpleNamespace(name="this.txt", path=path, sha256=sha256sum)


@pytest.fixture(scope="module")
def ssl_webserver(integration_files_dir, this_txt_file):
    """
    spins up an https webserver.
    """
    context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
    context.load_cert_chain(
        str(integration_files_dir / "https" / "cert.pem"),
        str(integration_files_dir / "https" / "key.pem"),
    )

    with Webserver(root=str(integration_files_dir), ssl_opts=context) as webserver:
        yield webserver


@pytest.fixture(scope="module")
def webserver(integration_files_dir, this_txt_file):
    """
    spins up an http webserver.
    """
    with Webserver(root=str(integration_files_dir)) as webserver:
        yield webserver


# ----- Async Test Fixtures ----------------------------------------------------------------------------------------->
# This is based on https://github.com/eukaryote/pytest-tornasync
# The reason why we don't use that pytest plugin instead is because it has
# tornado as a dependency, and we need to use the tornado we ship with salt


def get_test_timeout(pyfuncitem):
    default_timeout = 30
    marker = pyfuncitem.get_closest_marker("async_timeout")
    if marker:
        if marker.args:
            raise pytest.UsageError(
                "The 'async_timeout' marker does not accept any arguments "
                "only 'seconds' as a keyword argument"
            )
        return marker.kwargs.get("seconds") or default_timeout
    return default_timeout


@pytest.hookimpl(tryfirst=True)
def pytest_pycollect_makeitem(collector, name, obj):
    if collector.funcnamefilter(name) and inspect.iscoroutinefunction(obj):
        return list(collector._genfunctions(name, obj))


@pytest.hookimpl(tryfirst=True)
def pytest_runtest_setup(item):
    if inspect.iscoroutinefunction(item.obj):
        if "io_loop" not in item.fixturenames:
            # Append the io_loop fixture for the async functions
            item.fixturenames.append("io_loop")


class CoroTestFunction:
    def __init__(self, func, kwargs):
        self.func = func
        self.kwargs = kwargs
        functools.update_wrapper(self, func)

    async def __call__(self):
        ret = await self.func(**self.kwargs)
        return ret


@pytest.hookimpl(tryfirst=True)
def pytest_pyfunc_call(pyfuncitem):
    if not inspect.iscoroutinefunction(pyfuncitem.obj):
        return

    funcargs = pyfuncitem.funcargs
    testargs = {arg: funcargs[arg] for arg in pyfuncitem._fixtureinfo.argnames}

    try:
        loop = funcargs["io_loop"]
    except KeyError:
        loop = salt.ext.tornado.ioloop.IOLoop.current()

    __tracebackhide__ = True

    loop.run_sync(
        CoroTestFunction(pyfuncitem.obj, testargs), timeout=get_test_timeout(pyfuncitem)
    )
    return True


@pytest.fixture
def io_loop():
    """
    Create new io loop for each test, and tear it down after.
    """
    loop = salt.ext.tornado.ioloop.IOLoop()
    loop.make_current()
    try:
        yield loop
    finally:
        loop.clear_current()
        loop.close(all_fds=True)


# <---- Async Test Fixtures ------------------------------------------------------------------------------------------


# ----- Helpers ----------------------------------------------------------------------------------------------------->
@pytest.helpers.proxy.register
def delta_proxy_minion_ids():
    return [
        "dummy_proxy_one",
        "dummy_proxy_two",
        "dummy_proxy_three",
        "dummy_proxy_four",
    ]


# <---- Helpers ------------------------------------------------------------------------------------------------------



"""
unittests for json outputter
"""

import pytest

import salt.output.json_out as json_out
import salt.utils.stringutils
from tests.support.mock import patch


@pytest.fixture
def configure_loader_modules():
    return {json_out: {}}


@pytest.fixture
def data():
    return {"test": "two", "example": "one"}


def test_default_output(data):
    ret = json_out.output(data)
    assert '"test": "two"' in ret
    assert '"example": "one"' in ret


def test_pretty_output(data):
    with patch.dict(json_out.__opts__, {"output_indent": "pretty"}):
        ret = json_out.output(data)
        assert '"test": "two"' in ret
        assert '"example": "one"' in ret


def test_indent_output(data):
    with patch.dict(json_out.__opts__, {"output_indent": 2}):
        ret = json_out.output(data)
        assert '"test": "two"' in ret
        assert '"example": "one"' in ret


def test_negative_zero_output(data):
    with patch.dict(json_out.__opts__, {"output_indent": 0}):
        ret = json_out.output(data)
        assert '"test": "two"' in ret
        assert '"example": "one"' in ret


def test_negative_int_output(data):
    with patch.dict(json_out.__opts__, {"output_indent": -1}):
        ret = json_out.output(data)
        assert '"test": "two"' in ret
        assert '"example": "one"' in ret


def test_unicode_output():
    with patch.dict(json_out.__opts__, {"output_indent": "pretty"}):
        decoded = {"test": "Д", "example": "one"}
        encoded = {"test": salt.utils.stringutils.to_str("Д"), "example": "one"}
        # json.dumps on Python 2 adds a space before a newline while in the
        # process of dumping a dictionary.
        expected = '{\n    "example": "one",\n    "test": "Д"\n}'
        assert json_out.output(decoded) == expected
        assert json_out.output(encoded) == expected



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import pytest

import salt.modules.pyenv as pyenv
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {pyenv: {}}


def test_install():
    """
    Test if it install pyenv systemwide
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_ret = MagicMock(return_value=0)
    with patch.dict(
        pyenv.__salt__, {"config.option": mock_opt, "cmd.retcode": mock_ret}
    ):
        assert pyenv.install()


def test_update():
    """
    Test if it updates the current versions of pyenv and python-Build
    """
    mock_opt = MagicMock(return_value="salt stack")
    with patch.dict(pyenv.__salt__, {"config.option": mock_opt}):
        assert not pyenv.update()


def test_is_installed():
    """
    Test if it check if pyenv is installed.
    """
    mock_cmd = MagicMock(return_value=True)
    mock_opt = MagicMock(return_value="salt stack")
    with patch.dict(
        pyenv.__salt__, {"config.option": mock_opt, "cmd.has_exec": mock_cmd}
    ):
        assert pyenv.is_installed()


def test_install_python():
    """
    Test if it install a python implementation.
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_cmd = MagicMock(return_value=True)
    mock_all = MagicMock(
        return_value={"retcode": 0, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(pyenv.__grains__, {"os": "Linux"}):
        mock_all = MagicMock(
            return_value={"retcode": 0, "stdout": "salt", "stderr": "error"}
        )
        with patch.dict(
            pyenv.__salt__,
            {
                "config.option": mock_opt,
                "cmd.has_exec": mock_cmd,
                "cmd.run_all": mock_all,
            },
        ):
            assert pyenv.install_python("2.0.0-p0") == "error"

        mock_all = MagicMock(
            return_value={"retcode": True, "stdout": "salt", "stderr": "error"}
        )
        with patch.dict(
            pyenv.__salt__,
            {
                "config.option": mock_opt,
                "cmd.has_exec": mock_cmd,
                "cmd.run_all": mock_all,
            },
        ):
            assert not pyenv.install_python("2.0.0-p0")


def test_uninstall_python():
    """
    Test if it uninstall a python implementation.
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_cmd = MagicMock(return_value=True)
    mock_all = MagicMock(
        return_value={"retcode": True, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(
        pyenv.__salt__,
        {
            "cmd.has_exec": mock_cmd,
            "config.option": mock_opt,
            "cmd.run_all": mock_all,
        },
    ):
        assert pyenv.uninstall_python("2.0.0-p0")


def test_versions():
    """
    Test if it list the installed versions of python.
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_cmd = MagicMock(return_value=True)
    mock_all = MagicMock(
        return_value={"retcode": True, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(
        pyenv.__salt__,
        {
            "cmd.has_exec": mock_cmd,
            "config.option": mock_opt,
            "cmd.run_all": mock_all,
        },
    ):
        assert pyenv.versions() == []


def test_default():
    """
    Test if it returns or sets the currently defined default python.
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_cmd = MagicMock(return_value=True)
    mock_all = MagicMock(
        return_value={"retcode": True, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(
        pyenv.__salt__,
        {
            "cmd.has_exec": mock_cmd,
            "config.option": mock_opt,
            "cmd.run_all": mock_all,
        },
    ):
        assert pyenv.default() == ""
        assert pyenv.default("2.0.0-p0")


def test_list():
    """
    Test if it list the installable versions of python.
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_cmd = MagicMock(return_value=True)
    mock_all = MagicMock(
        return_value={"retcode": True, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(
        pyenv.__salt__,
        {
            "cmd.has_exec": mock_cmd,
            "config.option": mock_opt,
            "cmd.run_all": mock_all,
        },
    ):
        assert pyenv.list_() == []


def test_rehash():
    """
    Test if it run pyenv rehash to update the installed shims.
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_cmd = MagicMock(return_value=True)
    mock_all = MagicMock(
        return_value={"retcode": True, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(
        pyenv.__salt__,
        {
            "cmd.has_exec": mock_cmd,
            "config.option": mock_opt,
            "cmd.run_all": mock_all,
        },
    ):
        assert pyenv.rehash()


def test_do():
    """
    Test if it execute a python command with pyenv's
    shims from the user or the system.
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_cmd = MagicMock(return_value=True)
    mock_all = MagicMock(
        return_value={"retcode": True, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(
        pyenv.__salt__,
        {
            "cmd.has_exec": mock_cmd,
            "config.option": mock_opt,
            "cmd.run_all": mock_all,
        },
    ):
        assert not pyenv.do("gem list bundler")

    mock_all = MagicMock(
        return_value={"retcode": 0, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(
        pyenv.__salt__,
        {
            "config.option": mock_opt,
            "cmd.has_exec": mock_cmd,
            "cmd.run_all": mock_all,
        },
    ):
        assert pyenv.do("gem list bundler") == "salt"


def test_do_with_python():
    """
    Test if it execute a python command with pyenv's
    shims using a specific python version.
    """
    mock_opt = MagicMock(return_value="salt stack")
    mock_cmd = MagicMock(return_value=True)
    mock_all = MagicMock(
        return_value={"retcode": True, "stdout": "salt", "stderr": "error"}
    )
    with patch.dict(
        pyenv.__salt__,
        {
            "cmd.has_exec": mock_cmd,
            "config.option": mock_opt,
            "cmd.run_all": mock_all,
        },
    ):
        assert not pyenv.do_with_python("2.0.0-p0", "gem list bundler")



import salt.returners.sentry_return as sentry


def test_get_message():
    ret = {
        "id": "12345",
        "fun": "mytest.func",
        "fun_args": ["arg1", "arg2", {"foo": "bar"}],
        "jid": "54321",
        "return": "Long Return containing a Traceback",
    }

    assert sentry._get_message(ret) == "salt func: mytest.func arg1 arg2 foo=bar"
    assert (
        sentry._get_message({"fun": "test.func", "fun_args": []})
        == "salt func: test.func"
    )
    assert sentry._get_message({"fun": "test.func"}) == "salt func: test.func"



"""
Tests for file.rename state function
"""

# nox -e pytest-zeromq-3.8(coverage=False) -- -vvv --run-slow --run-destructive tests\pytests\functional\states\file\test_rename.py

import pytest

import salt.utils.path

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.slow_test,
]


@pytest.fixture(scope="module")
def file(states):
    return states.file


@pytest.fixture(scope="function")
def source():
    with pytest.helpers.temp_file(
        name="old_name.txt", contents="Source content"
    ) as source:
        yield source


def test_defaults(file, source):
    """
    Test file.rename with defaults
    """
    new_name = source.parent / "new_name.txt"
    try:
        file.rename(name=str(new_name), source=str(source))
        assert new_name.exists()
        assert not source.exists()
    finally:
        new_name.unlink()


def test_relative_name(file):
    """
    Test file.rename when name is a relative path
    """
    result = file.rename(name="..\\rel\\path\\test", source=str(source))
    assert "is not an absolute path" in result.filtered["comment"]
    assert result.filtered["result"] is False


def test_missing_source(file, source):
    """
    Test file.rename with the source file is missing
    """
    new_name = source.parent / "new_name.txt"
    missing_name = source.parent / "missing.txt"
    result = file.rename(name=str(new_name), source=str(missing_name))
    assert "has already been moved out of place" in result.filtered["comment"]
    assert result.filtered["result"] is True


def test_target_exists(file, source):
    """
    Test file.rename when there is an existing file with the new name
    """
    new_name = source.parent / "new_name.txt"
    new_name.write_text("existing file")
    try:
        result = file.rename(name=str(new_name), source=str(source))
        assert "exists and will not be overwritten" in result.filtered["comment"]
        assert result.filtered["result"] is True
    finally:
        new_name.unlink()


def test_target_exists_force(file, source):
    """
    Test file.rename when there is an existing file with the new name and
    force=True
    """
    new_name = source.parent / "new_name.txt"
    new_name.write_text("existing file")
    try:
        file.rename(name=str(new_name), source=str(source), force=True)
        assert new_name.exists()
        assert not source.exists()
        assert new_name.read_text() == "Source content"
    finally:
        new_name.unlink()


def test_test_is_true(file, source):
    new_name = source.parent / "new_name.txt"
    result = file.rename(name=str(new_name), source=str(source), test=True)
    assert "is set to be moved to" in result.filtered["comment"]
    assert result.filtered["result"] is None


def test_missing_dirs(file, source):
    new_name = source.parent / "missing_subdir" / "new_name.txt"
    result = file.rename(name=str(new_name), source=str(source))
    assert "is not present" in result.filtered["comment"]
    assert result.filtered["result"] is False


def test_missing_dirs_makedirs(file, source):
    new_name = source.parent / "missing_subdir" / "new_name.txt"
    try:
        file.rename(name=str(new_name), source=str(source), makedirs=True)
        assert new_name.exists()
        assert not source.exists()
    finally:
        new_name.unlink()
        new_name.parent.rmdir()


def test_source_is_link(file, source):
    link_source = source.parent / "link_source.lnk"
    link_source.symlink_to(source)
    new_name = source.parent / "new_name.lnk"
    try:
        file.rename(name=str(new_name), source=str(link_source))
        assert new_name.exists()
        assert new_name.is_symlink()
        assert salt.utils.path.readlink(str(new_name)) == str(source)
        assert new_name.read_text() == "Source content"
        assert not link_source.exists()
    finally:
        new_name.unlink()



"""
Yaml helper module for troubleshooting yaml


.. versionadded:: 3005

:depends:   yamllint >= 1.20.0


"""

import logging

import salt.utils.versions

log = logging.getLogger(__name__)

__virtualname__ = "yaml"


try:
    import salt.utils.yamllint

    HAS_LINT = True
except ImportError:
    HAS_LINT = False


def __virtual__():

    if HAS_LINT:
        if salt.utils.yamllint.has_yamllint():
            version = salt.utils.yamllint.version()
            version_cmp = salt.utils.versions.version_cmp(version, "1.20.0")
        else:
            return (False, "yamllint not installed")
        if version_cmp >= 0:
            return __virtualname__
        else:
            return (False, "yamllint below 1.20.0, please pip install a newer version")
    else:
        return (False, "yamllint not installed")


def lint(source, saltenv=None, pre_render=None, **kwargs):
    """
    lint the output after detecting a sucsessful render.

    :param str source: managed source file

    :param str saltenv: the saltenv to use, defaults
        to minions enviroment or base if not set

    :param str pre_render: The render options passed to
        slsutil.renderer other wise file is cached and loaded as stream

    CLI Example:

    .. code-block:: bash

        salt '*' yaml.lint salt://example/bad_yaml.sls
    """
    if saltenv is None:
        saltenv = __salt__["config.get"]("saltenv", "base")
        if saltenv is None:
            saltenv = "base"
    if pre_render is None:
        cache = __salt__["cp.cache_file"](source, saltenv)
        if cache is False:
            return (False, "Template was unable to be cached")
        with __utils__["files.fopen"](cache, "r") as yaml_stream:
            yaml_out = yaml_stream.read(-1)
    else:
        kwargs.update({"saltenv": saltenv})
        yaml_out = __salt__["slsutil.renderer"](
            path=source, default_renderer=pre_render, **kwargs
        )
    return salt.utils.yamllint.lint(yaml_out)



import tempfile

import pytest


@pytest.fixture(scope="module")
def minion_config_overrides():
    with tempfile.TemporaryDirectory() as tempdir:
        yield {
            "mydude": {
                "driver": "sqlite3",
                "database": tempdir + "/test_sdb.sq3",
                "table": __name__,
                "create_table": True,
            }
        }


@pytest.mark.parametrize(
    "expected_value",
    (
        "foo",
        b"bang",
        ["cool", b"guy", "dude", b"\x00\x31\x99\x42"],
        {
            "this": b"has some",
            b"complicated": "things",
            "all": [{"going": "on"}, {"but": "that", 42: "should be fine"}],
        },
    ),
)
def test_setting_sdb_values_with_text_and_bytes_should_retain_data_types(
    expected_value, modules
):
    modules.sdb.set("sdb://mydude/fnord", expected_value)
    actual_value = modules.sdb.get("sdb://mydude/fnord", strict=True)
    assert actual_value == expected_value



import json
import logging
import time

import pytest

from tests.conftest import CODE_DIR

docker = pytest.importorskip("docker")

log = logging.getLogger(__name__)

pytestmark = [
    pytest.mark.core_test,
    pytest.mark.timeout_unless_on_windows(600),
]


def json_output_to_dict(output):
    r"""
    Convert ``salt ... --out=json`` Syndic return to a dictionary. Since the
    --out=json will return several JSON outputs, e.g. {...}\n{...}, we have to
    parse that output individually.
    """
    output = output or ""
    results = {}
    for line in (
        _ for _ in output.replace("\n}", "\n}\x1f").split("\x1f") if _.strip()
    ):
        data = json.loads(line)
        if isinstance(data, dict):
            for minion in data:
                # Filter out syndic minions since they won't show up in the future
                if minion not in ("syndic_a", "syndic_b"):
                    results[minion] = data[minion]
    return results


@pytest.fixture(scope="module")
def syndic_network():
    try:
        client = docker.from_env()
    except docker.errors.DockerException as e:
        # Docker failed, it's gonna be an environment issue, let's just skip
        pytest.skip(f"Docker failed with error {e}")
    pool = docker.types.IPAMPool(
        subnet="172.27.13.0/24",
        gateway="172.27.13.1",
    )
    ipam_config = docker.types.IPAMConfig(
        pool_configs=[pool],
    )
    network = None
    try:
        network = client.networks.create(name="syndic_test_net", ipam=ipam_config)
        yield network.name
    finally:
        if network is not None:
            network.remove()


@pytest.fixture(scope="module")
def container_image_name():
    return "ghcr.io/saltstack/salt-ci-containers/salt:3006"


@pytest.fixture(scope="module")
def container_python_version():
    return "3.10"


@pytest.fixture(scope="module")
def config(tmp_path_factory):
    tmp_path = tmp_path_factory.mktemp("eauth")
    master_dir = tmp_path / "master"
    minion_dir = tmp_path / "minion"
    syndic_a_dir = tmp_path / "syndic_a"
    syndic_b_dir = tmp_path / "syndic_b"
    minion_a1_dir = tmp_path / "minion_a1"
    minion_a2_dir = tmp_path / "minion_a2"
    minion_b1_dir = tmp_path / "minion_b1"
    minion_b2_dir = tmp_path / "minion_b2"

    for dir_ in (
        master_dir,
        minion_dir,
        syndic_a_dir,
        syndic_b_dir,
        minion_a1_dir,
        minion_a2_dir,
        minion_b1_dir,
        minion_b2_dir,
    ):
        dir_.mkdir(parents=True, exist_ok=True)
        (dir_ / "master.d").mkdir(exist_ok=True)
        # minion.d is probably needed to prevent errors on tempdir cleanup
        (dir_ / "minion.d").mkdir(exist_ok=True)
        (dir_ / "pki").mkdir(exist_ok=True)
    (master_dir / "master.d").mkdir(exist_ok=True)

    master_config_path = master_dir / "master"
    master_config_path.write_text(
        """
open_mode: True
auth.pam.python: /usr/local/bin/python3
order_masters: True

publisher_acl:
  bob:
    - '*1':
      - test.*
      - file.touch

external_auth:
  pam:
    bob:
      - '*1':
        - test.*
        - file.touch

nodegroups:
  second_string: "minion_*2"
  b_string: "minion_b*"

    """
    )

    minion_config_path = minion_dir / "minion"
    minion_config_path.write_text("id: minion\nmaster: master\nopen_mode: True")

    syndic_a_minion_config_path = syndic_a_dir / "minion"
    syndic_a_minion_config_path.write_text(
        "id: syndic_a\nmaster: master\nopen_mode: True"
    )
    syndic_a_master_config_path = syndic_a_dir / "master"
    syndic_a_master_config_path.write_text(
        """
open_mode: True
auth.pam.python: /usr/local/bin/python3
syndic_master: master
publisher_acl:
  bob:
    - '*1':
      - test.*
      - file.touch

external_auth:
  pam:
    bob:
      - '*1':
        - test.*
        - file.touch
    """
    )

    minion_a1_config_path = minion_a1_dir / "minion"
    minion_a1_config_path.write_text("id: minion_a1\nmaster: syndic_a\nopen_mode: True")
    minion_a2_config_path = minion_a2_dir / "minion"
    minion_a2_config_path.write_text("id: minion_a2\nmaster: syndic_a\nopen_mode: True")

    syndic_b_minion_config_path = syndic_b_dir / "minion"
    syndic_b_minion_config_path.write_text(
        "id: syndic_b\nmaster: master\nopen_mode: True"
    )
    syndic_b_master_config_path = syndic_b_dir / "master"
    syndic_b_master_config_path.write_text("syndic_master: master\nopen_mode: True")

    minion_b1_config_path = minion_b1_dir / "minion"
    minion_b1_config_path.write_text("id: minion_b1\nmaster: syndic_b\nopen_mode: True")
    minion_b2_config_path = minion_b2_dir / "minion"
    minion_b2_config_path.write_text("id: minion_b2\nmaster: syndic_b\nopen_mode: True")

    return {
        "minion_dir": minion_dir,
        "master_dir": master_dir,
        "syndic_a_dir": syndic_a_dir,
        "syndic_b_dir": syndic_b_dir,
        "minion_a1_dir": minion_a1_dir,
        "minion_a2_dir": minion_a2_dir,
        "minion_b1_dir": minion_b1_dir,
        "minion_b2_dir": minion_b2_dir,
    }


@pytest.fixture(scope="module")
def docker_master(
    salt_factories,
    syndic_network,
    config,
    container_image_name,
    container_python_version,
):
    config_dir = str(config["master_dir"])
    container = salt_factories.get_container(
        "master",
        image_name=container_image_name,
        container_run_kwargs={
            # "entrypoint": "salt-master -ldebug",
            "entrypoint": "python -m http.server",
            "network": syndic_network,
            "volumes": {
                config_dir: {"bind": "/etc/salt", "mode": "z"},
                str(CODE_DIR / "salt"): {
                    "bind": f"/usr/local/lib/python{container_python_version}/site-packages/salt/",
                    "mode": "z",
                },
            },
        },
        pull_before_start=True,
        skip_on_pull_failure=True,
        skip_if_docker_client_not_connectable=True,
    )
    # container.container_start_check(confirm_container_started, container)
    with container.started() as factory:
        for user in ("bob", "fnord"):
            ret = container.run(f"adduser {user}")
            assert ret.returncode == 0
            ret = container.run(f"passwd -d {user}")
            assert ret.returncode == 0
        yield factory


@pytest.fixture(scope="module")
def docker_minion(
    salt_factories,
    syndic_network,
    config,
    container_image_name,
    container_python_version,
):
    config_dir = str(config["minion_dir"])
    container = salt_factories.get_container(
        "minion",
        image_name=container_image_name,
        container_run_kwargs={
            # "entrypoint": "salt-minion",
            "entrypoint": "python -m http.server",
            "network": syndic_network,
            "volumes": {
                config_dir: {"bind": "/etc/salt", "mode": "z"},
                str(CODE_DIR / "salt"): {
                    "bind": f"/usr/local/lib/python{container_python_version}/site-packages/salt/",
                    "mode": "z",
                },
            },
        },
        pull_before_start=True,
        skip_on_pull_failure=True,
        skip_if_docker_client_not_connectable=True,
    )
    # container.container_start_check(confirm_container_started, container)
    with container.started() as factory:
        yield factory


@pytest.fixture(scope="module")
def docker_syndic_a(
    salt_factories,
    config,
    syndic_network,
    container_image_name,
    container_python_version,
):
    config_dir = str(config["syndic_a_dir"])
    container = salt_factories.get_container(
        "syndic_a",
        image_name=container_image_name,
        container_run_kwargs={
            # "entrypoint": "salt-master -ldebug",
            "entrypoint": "python -m http.server",
            "network": syndic_network,
            "volumes": {
                config_dir: {"bind": "/etc/salt", "mode": "z"},
                str(CODE_DIR / "salt"): {
                    "bind": f"/usr/local/lib/python{container_python_version}/site-packages/salt/",
                    "mode": "z",
                },
            },
        },
        pull_before_start=True,
        skip_on_pull_failure=True,
        skip_if_docker_client_not_connectable=True,
    )
    # container.container_start_check(confirm_container_started, container)
    with container.started() as factory:
        yield factory


@pytest.fixture(scope="module")
def docker_syndic_b(
    salt_factories,
    config,
    syndic_network,
    container_image_name,
    container_python_version,
):
    config_dir = str(config["syndic_b_dir"])
    container = salt_factories.get_container(
        "syndic_b",
        image_name=container_image_name,
        container_run_kwargs={
            # "entrypoint": "salt-master -ldebug",
            "entrypoint": "python -m http.server",
            "network": syndic_network,
            "volumes": {
                config_dir: {"bind": "/etc/salt", "mode": "z"},
                str(CODE_DIR / "salt"): {
                    "bind": f"/usr/local/lib/python{container_python_version}/site-packages/salt/",
                    "mode": "z",
                },
            },
        },
        pull_before_start=True,
        skip_on_pull_failure=True,
        skip_if_docker_client_not_connectable=True,
    )
    # container.container_start_check(confirm_container_started, container)
    with container.started() as factory:
        yield factory


@pytest.fixture(scope="module")
def docker_minion_a1(
    salt_factories,
    config,
    syndic_network,
    container_image_name,
    container_python_version,
):
    config_dir = str(config["minion_a1_dir"])
    container = salt_factories.get_container(
        "minion_a1",
        image_name=container_image_name,
        container_run_kwargs={
            "network": syndic_network,
            # "entrypoint": "salt-minion -ldebug",
            "entrypoint": "python -m http.server",
            "volumes": {
                config_dir: {"bind": "/etc/salt", "mode": "z"},
                str(CODE_DIR / "salt"): {
                    "bind": f"/usr/local/lib/python{container_python_version}/site-packages/salt/",
                    "mode": "z",
                },
            },
        },
        pull_before_start=True,
        skip_on_pull_failure=True,
        skip_if_docker_client_not_connectable=True,
    )
    # container.container_start_check(confirm_container_started, container)
    with container.started() as factory:
        yield factory


@pytest.fixture(scope="module")
def docker_minion_a2(
    salt_factories,
    config,
    syndic_network,
    container_image_name,
    container_python_version,
):
    config_dir = str(config["minion_a2_dir"])
    container = salt_factories.get_container(
        "minion_a2",
        image_name=container_image_name,
        container_run_kwargs={
            "network": syndic_network,
            # "entrypoint": "salt-minion",
            "entrypoint": "python -m http.server",
            "volumes": {
                config_dir: {"bind": "/etc/salt", "mode": "z"},
                str(CODE_DIR / "salt"): {
                    "bind": f"/usr/local/lib/python{container_python_version}/site-packages/salt/",
                    "mode": "z",
                },
            },
        },
        pull_before_start=True,
        skip_on_pull_failure=True,
        skip_if_docker_client_not_connectable=True,
    )
    # container.container_start_check(confirm_container_started, container)
    with container.started() as factory:
        yield factory


@pytest.fixture(scope="module")
def docker_minion_b1(
    salt_factories,
    config,
    syndic_network,
    container_image_name,
    container_python_version,
):
    config_dir = str(config["minion_b1_dir"])
    container = salt_factories.get_container(
        "minion_b1",
        image_name=container_image_name,
        container_run_kwargs={
            "network": syndic_network,
            # "entrypoint": "salt-minion",
            "entrypoint": "python -m http.server",
            "volumes": {
                config_dir: {"bind": "/etc/salt", "mode": "z"},
                str(CODE_DIR / "salt"): {
                    "bind": f"/usr/local/lib/python{container_python_version}/site-packages/salt/",
                    "mode": "z",
                },
            },
        },
        pull_before_start=True,
        skip_on_pull_failure=True,
        skip_if_docker_client_not_connectable=True,
    )
    # container.container_start_check(confirm_container_started, container)
    with container.started() as factory:
        yield factory


@pytest.fixture(scope="module")
def docker_minion_b2(
    salt_factories,
    config,
    syndic_network,
    container_image_name,
    container_python_version,
):
    config_dir = str(config["minion_b2_dir"])
    container = salt_factories.get_container(
        "minion_b2",
        image_name=container_image_name,
        container_run_kwargs={
            "network": syndic_network,
            # "entrypoint": "salt-minion",
            "entrypoint": "python -m http.server",
            "volumes": {
                config_dir: {"bind": "/etc/salt", "mode": "z"},
                str(CODE_DIR / "salt"): {
                    "bind": f"/usr/local/lib/python{container_python_version}/site-packages/salt/",
                    "mode": "z",
                },
            },
        },
        pull_before_start=True,
        skip_on_pull_failure=True,
        skip_if_docker_client_not_connectable=True,
    )
    # container.container_start_check(confirm_container_started, container)
    with container.started() as factory:
        yield factory


@pytest.fixture(scope="module", autouse=True)
def all_the_docker(
    docker_master,
    docker_minion,
    docker_syndic_a,
    docker_syndic_b,
    docker_minion_a1,
    docker_minion_a2,
    docker_minion_b1,
    docker_minion_b2,
):
    try:
        for s in (
            docker_master,
            docker_syndic_a,
            docker_syndic_b,
            docker_minion_a1,
            docker_minion_a2,
            docker_minion_b1,
            docker_minion_b2,
            docker_minion,
        ):
            s.run("python3 -m pip install looseversion packaging")
        # WORKAROUND
        for s in (docker_master, docker_syndic_a, docker_syndic_b):
            s.run("salt-master -d -ldebug")
        for s in (
            docker_minion_a1,
            docker_minion_a2,
            docker_minion_b1,
            docker_minion_b2,
            docker_minion,
        ):
            s.run("salt-minion -d")
        # END WORKAROUND
        for s in (docker_syndic_a, docker_syndic_b):
            s.run("salt-syndic -d")
        failure_time = time.time() + (5 * 60)
        results = None
        while time.time() < failure_time:
            res = docker_master.run(r"salt \* test.ping -t10 --out=json")
            results = json_output_to_dict(res.stdout)
            if set(results).issuperset(
                ["minion", "minion_a1", "minion_a2", "minion_b1", "minion_b2"]
            ):
                break
            time.sleep(5)
        else:
            pytest.skip(f"Missing some minions: {sorted(results)}")

        yield
    finally:
        # looks like we need to do this to actually let the test suite run as non-root.
        for container in (
            docker_minion,
            docker_syndic_a,
            docker_syndic_b,
            docker_minion_a1,
            docker_minion_a2,
            docker_minion_b1,
            docker_minion_b2,
        ):
            try:
                container.run("rm -rfv /etc/salt/")
                # If you need to debug this ^^^^^^^
                # use this vvvvvv
                # res = container.run('rm -rfv /etc/salt/')
                # print(container)
                # print(res.stdout)
            except docker.errors.APIError as exc:
                # if the container isn't running, there's not thing we can do
                # at this point.
                log.info("Docker failed removing /etc/salt: %s", exc)


@pytest.fixture(
    params=[
        ("*", ["minion", "minion_a1", "minion_a2", "minion_b1", "minion_b2"]),
        ("minion", ["minion"]),
        ("minion_*", ["minion_a1", "minion_a2", "minion_b1", "minion_b2"]),
        ("minion_a*", ["minion_a1", "minion_a2"]),
        ("minion_b*", ["minion_b1", "minion_b2"]),
        ("*1", ["minion_a1", "minion_b1"]),
        ("*2", ["minion_a2", "minion_b2"]),
    ]
)
def all_the_minions(request):
    yield request.param


@pytest.fixture(
    params=[
        ("minion_a1", ["minion_a1"]),
        ("minion_b1", ["minion_b1"]),
        ("*1", ["minion_a1", "minion_b1"]),
        ("minion*1", ["minion_a1", "minion_b1"]),
    ]
)
def eauth_valid_minions(request):
    yield request.param


@pytest.fixture(
    params=[
        "*",
        "minion",
        "minion_a2",
        "minion_b2",
        "syndic_a",
        "syndic_b",
        "*2",
        "minion*",
        "minion_a*",
        "minion_b*",
    ]
)
def eauth_blocked_minions(request):
    yield request.param


@pytest.fixture
def docker_minions(
    docker_minion,
    docker_minion_a1,
    docker_minion_a2,
    docker_minion_b1,
    docker_minion_b2,
):
    yield [
        docker_minion,
        docker_minion_a1,
        docker_minion_a2,
        docker_minion_b1,
        docker_minion_b2,
    ]


@pytest.fixture(
    params=[
        "test.arg good_argument",
        "test.arg bad_news",
        "test.arg not_allowed",
        "test.echo very_not_good",
        "cmd.run 'touch /tmp/fun.txt'",
        "file.touch /tmp/more_fun.txt",
        "test.arg_repr this_is_whatever",
        "test.arg_repr more whatever",
        "test.arg_repr cool guy",
    ]
)
def all_the_commands(request):
    yield request.param


@pytest.fixture(
    params=[
        "test.arg",
        "test.echo",
    ]
)
def eauth_valid_commands(request):
    yield request.param


@pytest.fixture(
    params=[
        "cmd.run",
        "file.manage_file",
        "test.arg_repr",
    ]
)
def eauth_invalid_commands(request):
    yield request.param


@pytest.fixture(
    params=[
        "good_argument",
        "good_things",
        "good_super_awesome_stuff",
    ]
)
def eauth_valid_arguments(request):
    # TODO: Should these be part of valid commands? I don't know yet! -W. Werner, 2022-12-01
    yield request.param


@pytest.fixture(
    params=[
        "bad_news",
        "not_allowed",
        "very_not_good",
    ]
)
def eauth_invalid_arguments(request):
    yield request.param


@pytest.fixture(
    params=[
        "G@id:minion_a1 and minion_b*",
        "E@minion_[^b]1 and minion_b2",
        "P@id:minion_[^b]. and minion",
        # TODO: Do soemthing better with these. Apparently lists work... weirdly -W. Werner, 2022-12-02
        # "L@minion_* not L@minion_*2 and minion",
        # "I@has_syndic:* not minion_b2 not minion_a2 and minion",
        # "J@has_syndic:syndic_(a|b) not *2 and minion",
        # TODO: I need to figure out a good way to get IPs -W. Werner, 2022-12-02
        # "S@172.13.1.4 and S@172.13.1.5 and minion_*2",
        # TODO:  I don't have any range servers -W. Werner, 2022-12-02
        # "((R@%minion_a1..2 and R@%minion_b1..2) not N@second_string) and minion",
    ]
)
def invalid_comprehensive_minion_targeting(request):
    yield request.param


@pytest.fixture(
    params=[
        (
            "G@id:minion or minion_a1 or E@minion_[^b]2 or L@minion_b1,minion_b2",
            ["minion", "minion_a1", "minion_a2", "minion_b1", "minion_b2"],
        ),
        (
            "minion or E@minion_a[12] or N@b_string",
            ["minion", "minion_a1", "minion_a2", "minion_b1", "minion_b2"],
        ),
        (
            "L@minion,minion_a1 or N@second_string or N@b_string",
            ["minion", "minion_a1", "minion_a2", "minion_b1", "minion_b2"],
        ),
        # TODO: I don't have pillar setup, nor do I know IPs, and also SECO range servers -W. Werner, 2022-12-02
        #        (
        #            "I@my_minion:minion and I@has_syndic:syndic_[^b] and S@172.13.1.4 and S@172.13.1.5",
        #            ["minion", "minion_a1", "minion_a2", "minion_b1", "minion_b2"],
        #        ),
        #        (
        #            "minion and R@%minion_a1..2 and N@b_string",
        #            ["minion", "minion_a1", "minion_a2", "minion_b1", "minion_b2"],
        #        ),
    ]
)
def comprehensive_minion_targeting(request):
    # TODO: include SECO range? -W. Werner, 2022-12-01
    yield request.param


@pytest.fixture(
    params=[
        ("G@id:minion_a1 and minion_b1", ["minion_a1", "minion_b1"]),
        ("E@minion_[^b]1", ["minion_a1"]),
        (
            "P@id:minion_[^b].",
            ["minion_a1", "minion_a2"],
        ),  # should this be this thing? or something different?
        # TODO: Turns out that it doesn't exclude things? Not sure -W. Werner, 2022-12-02
        (
            "L@minion_a1,minion_a2,minion_b1 not minion_*2",
            ["minion_a1", "minion_a2", "minion_b1"],
        ),
        # TODO: I don't have pillar data yet -W. Werner, 2022-12-02
        # ("I@has_syndic:* not minion_b2 not minion_a2", ["minion_a1", "minion_b1"]),
        # ("J@has_syndic:syndic_(a|b) not *2", ["minion_a1", "minion_b1"]),
        # TODO: Need a different way to get IPs -W. Werner, 2022-12-02
        # ( "S@172.13.1.4 and S@172.13.1.5", ["minion_a1", "minion_b1"]),
        # TODO: Need a range server for these tests (see range targeting docs) -W. Werner, 2022-12-02
        # ("(R@%minion_a1..2 and R@%minion_b1..2) not N@second_string", ["minion_a1", "minion_b1"]),
    ]
)
def valid_comprehensive_minion_targeting(request):
    yield request.param


@pytest.fixture(
    params=[
        # TODO: not sure why this doesn't work. Pretty sure it's just the cli part -W. Werner, 2022-12-02
        # ("G@id:minion_a1 and minion_b1", ["minion_a1", "minion_b1"]),
        ("E@minion_[^b]1", ["minion_a1"]),
        (
            "P@id:minion_[^a]1",
            ["minion_b1"],
        ),
        ("L@minion_a1,minion_b1 not minion_*2", ["minion_a1", "minion_b1"]),
        # TODO: need to add pillars -W. Werner, 2022-12-02
        # ("I@has_syndic:* not minion_b2 not minion_a2", ["minion_a1", "minion_b1"]),
        # ("J@has_syndic:syndic_(a|b) not *2", ["minion_a1", "minion_b1"]),
        # TODO: Need a different way to get IPs -W. Werner, 2022-12-02
        # ( "S@172.13.1.4 and S@172.13.1.5", ["minion_a1", "minion_b1"]),
        # TODO: Need a range server for these tests (see range targeting docs) -W. Werner, 2022-12-02
        # ("(R@%minion_a1..2 and R@%minion_b1..2) not N@second_string", ["minion_a1", "minion_b1"]),
    ]
)
def valid_eauth_comprehensive_minion_targeting(request):
    yield request.param


def test_root_user_should_be_able_to_call_any_and_all_minions_with_any_and_all_commands(
    all_the_minions, all_the_commands, docker_master
):
    target, expected_minions = all_the_minions
    res = docker_master.run(
        f"salt {target} {all_the_commands} -t 10 --out=json",
    )
    if "jid does not exist" in (res.stderr or ""):
        # might be flaky, let's retry
        res = docker_master.run(
            f"salt {target} {all_the_commands} -t 10 --out=json",
        )
    results = json_output_to_dict(res.stdout)
    assert sorted(results) == expected_minions, res.stdout


def test_eauth_user_should_be_able_to_target_valid_minions_with_valid_command(
    eauth_valid_minions, eauth_valid_commands, eauth_valid_arguments, docker_master
):
    target, expected_minions = eauth_valid_minions
    res = docker_master.run(
        f"salt -a pam --username bob --password '' {target} {eauth_valid_commands} {eauth_valid_arguments} -t 10 --out=json",
    )
    results = json_output_to_dict(res.stdout)
    assert sorted(results) == expected_minions, res.stdout


def test_eauth_user_should_not_be_able_to_target_invalid_minions(
    eauth_blocked_minions, docker_master, docker_minions
):
    res = docker_master.run(
        f"salt -a pam --username bob --password '' {eauth_blocked_minions} file.touch /tmp/bad_bad_file.txt -t 10 --out=json",
    )
    assert "Authorization error occurred." == res.data or res.data is None
    for minion in docker_minions:
        res = minion.run("test -f /tmp/bad_bad_file.txt")
        file_exists = res.returncode == 0
        assert not file_exists


@pytest.mark.skip(reason="Not sure about blocklist")
def test_eauth_user_should_not_be_able_to_target_valid_minions_with_invalid_commands(
    eauth_valid_minions, eauth_invalid_commands, docker_master
):
    tgt, _ = eauth_valid_minions
    res = docker_master.run(
        f"salt -a pam --username bob --password '' {tgt} {eauth_invalid_commands} -t 10 --out=json",
    )
    results = json_output_to_dict(res.stdout)
    assert "Authorization error occurred" in res.stdout
    assert sorted(results) == []


@pytest.mark.skip(reason="Not sure about blocklist")
def test_eauth_user_should_not_be_able_to_target_valid_minions_with_valid_commands_and_invalid_arguments(
    eauth_valid_minions, eauth_valid_commands, eauth_invalid_arguments, docker_master
):
    tgt, _ = eauth_valid_minions
    res = docker_master.run(
        f"salt -a pam --username bob --password '' -C '{tgt}' {eauth_valid_commands} {eauth_invalid_arguments} -t 10 --out=json",
    )
    results = json_output_to_dict(res.stdout)
    assert "Authorization error occurred" in res.stdout
    assert sorted(results) == []


def test_invalid_eauth_user_should_not_be_able_to_do_anything(
    eauth_valid_minions, eauth_valid_commands, eauth_valid_arguments, docker_master
):
    # TODO: Do we really need to run all of these tests for the invalid user? Maybe not! -W. Werner, 2022-12-01
    tgt, _ = eauth_valid_minions
    res = docker_master.run(
        f"salt -a pam --username badguy --password '' -C '{tgt}' {eauth_valid_commands} {eauth_valid_arguments} -t 10 --out=json",
    )
    results = json_output_to_dict(res.stdout)
    assert sorted(results) == []


def test_root_should_be_able_to_use_comprehensive_targeting(
    comprehensive_minion_targeting, docker_master
):
    tgt, expected_minions = comprehensive_minion_targeting
    res = docker_master.run(
        f"salt -C '{tgt}' test.version -t 10 --out=json",
    )
    results = json_output_to_dict(res.stdout)
    assert sorted(results) == expected_minions


def test_eauth_user_should_be_able_to_target_valid_minions_with_valid_commands_comprehensively(
    valid_eauth_comprehensive_minion_targeting, docker_master
):
    tgt, expected_minions = valid_eauth_comprehensive_minion_targeting
    res = docker_master.run(
        f"salt -a pam --username bob --password '' -C '{tgt}' test.version -t 10 --out=json",
    )
    results = json_output_to_dict(res.stdout)
    assert sorted(results) == expected_minions


def test_eauth_user_with_invalid_comprehensive_targeting_should_auth_failure(
    invalid_comprehensive_minion_targeting, docker_master
):
    res = docker_master.run(
        f"salt -a pam --username fnord --password '' -C '{invalid_comprehensive_minion_targeting}' test.version -t 10 --out=json",
    )
    results = json_output_to_dict(res.stdout)
    assert "Authorization error occurred" in res.stdout
    assert sorted(results) == []



"""
    :codeauthor: Megan Wilhite <mwilhite@saltstack.com>
"""

import logging
import socket

from tests.integration.cloud.helpers.cloud_test_base import TIMEOUT, CloudTest
from tests.support.helpers import TstSuiteLoggingHandler


class VMWareTest(CloudTest):
    """
    Integration tests for the vmware cloud provider in Salt-Cloud
    """

    PROVIDER = "vmware"
    REQUIRED_PROVIDER_CONFIG_ITEMS = ("password", "user", "url")

    def setUp(self):
        super().setUp()
        test_host = self.provider_config["url"]
        try:
            socket.gethostbyname_ex(test_host)
        except OSError as exc:
            self.skipTest(
                "The required vmware host at {} is not available: {}".format(
                    test_host, exc
                )
            )

    def test_instance(self):
        """
        Tests creating and deleting an instance on vmware and installing salt
        """
        # create the instance
        disk_datastore = self.config["vmware-test"]["devices"]["disk"]["Hard disk 2"][
            "datastore"
        ]

        ret_val = self.run_cloud(
            f"-p vmware-test {self.instance_name}", timeout=TIMEOUT
        )
        disk_datastore_str = "                [{}] {}/Hard disk 2-flat.vmdk".format(
            disk_datastore, self.instance_name
        )

        # check if instance returned with salt installed
        self.assertInstanceExists(ret_val)
        self.assertIn(
            disk_datastore_str,
            ret_val,
            msg=f"Hard Disk 2 did not use the Datastore {disk_datastore} ",
        )

        self.assertDestroyInstance()

    def test_snapshot(self):
        """
        Tests creating snapshot and creating vm with --no-deploy
        """
        # create the instance
        ret_val = self.run_cloud(
            f"-p vmware-test {self.instance_name} --no-deploy", timeout=TIMEOUT
        )

        # check if instance returned with salt installed
        self.assertInstanceExists(ret_val)

        create_snapshot = self.run_cloud(
            "-a create_snapshot {} snapshot_name='Test Cloud' memdump=True -y".format(
                self.instance_name
            ),
            timeout=TIMEOUT,
        )
        s_ret_str = "Snapshot created successfully"

        self.assertIn(s_ret_str, str(create_snapshot))

        self.assertDestroyInstance()

    def test_verify_ssl_false(self):
        """
        Tests creating and deleting an instance on vmware when using
        verify_ssl: False
        """
        profile_name = "vmware_verify_ssl"
        self.add_profile_config(
            "vmware-test", {"verify_ssl": False}, "vmware.conf", profile_name
        )
        # create the instance
        ret_val = self.run_cloud(
            f"-p {profile_name} {self.instance_name}", timeout=TIMEOUT
        )
        # check if instance returned with salt installed
        self.assertInstanceExists(ret_val)
        self.assertDestroyInstance()

    def test_instant_clone(self):
        """
        Tests creating Instant Clone VM
        """
        # salt-cloud -p my-instant-clone IC3
        profile_name = "vmware-test-instant-clone"
        self.run_cloud("-a remove_all_snapshots cloud-tests-template-base")

        # create the instance
        log_format = "%(message)s"
        handler = TstSuiteLoggingHandler(format=log_format, level=logging.INFO)
        with handler:
            ret_val = self.run_cloud(
                f"-p {profile_name} {self.instance_name}", timeout=TIMEOUT
            )
            # This sometimes times out before it get's an IP, so we check the logs
            if ret_val == []:
                check_log = "Successfully completed Instantclone task"
                self.assertTrue(any(check_log in s for s in handler.messages))
            else:
                i_clone_str = "Instant Clone created successfully"
                self.assertIn(i_clone_str, str(ret_val))

        self.assertDestroyInstance()



import re

import pytest

from tests.support.case import ModuleCase


@pytest.mark.skip_unless_on_windows
class WinIPTest(ModuleCase):
    """
    Tests for salt.modules.win_ip
    """

    def test_get_default_gateway(self):
        """
        Test getting default gateway
        """
        ip = re.compile(r"^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
        ret = self.run_function("ip.get_default_gateway")
        assert ip.match(ret)

    def test_ip_is_enabled(self):
        """
        Test ip.is_enabled
        """
        assert self.run_function("ip.is_enabled", ["Ethernet"])
        assert "not found" in self.run_function("ip.is_enabled", ["doesnotexist"])



"""
:codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

import pytest

import salt.modules.win_service as win_service
import salt.utils.path
from salt.exceptions import CommandExecutionError
from tests.support.mock import MagicMock, patch

try:
    import pywintypes
    import win32serviceutil

    WINAPI = True
except ImportError:
    WINAPI = False

pytestmark = [
    pytest.mark.skip_unless_on_windows,
]


@pytest.fixture
def configure_loader_modules():
    return {win_service: {}}


def test_get_enabled():
    """
    Test to return the enabled services
    """
    mock = MagicMock(
        return_value=[
            {"ServiceName": "spongebob"},
            {"ServiceName": "squarepants"},
            {"ServiceName": "patrick"},
        ]
    )
    with patch.object(win_service, "_get_services", mock):
        mock_info = MagicMock(
            side_effect=[
                {"StartType": "Auto"},
                {"StartType": "Manual"},
                {"StartType": "Disabled"},
            ]
        )
        with patch.object(win_service, "info", mock_info):
            assert win_service.get_enabled() == ["spongebob"]


def test_get_disabled():
    """
    Test to return the disabled services
    """
    mock = MagicMock(
        return_value=[
            {"ServiceName": "spongebob"},
            {"ServiceName": "squarepants"},
            {"ServiceName": "patrick"},
        ]
    )
    with patch.object(win_service, "_get_services", mock):
        mock_info = MagicMock(
            side_effect=[
                {"StartType": "Auto"},
                {"StartType": "Manual"},
                {"StartType": "Disabled"},
            ]
        )
        with patch.object(win_service, "info", mock_info):
            result = win_service.get_disabled()
            expected = ["patrick", "squarepants"]
            assert result == expected


def test_available():
    """
    Test to Returns ``True`` if the specified service
    is available, otherwise returns ``False``
    """
    mock = MagicMock(return_value=["c", "a", "b"])
    with patch.object(win_service, "get_all", mock):
        assert win_service.available("a") is True


def test_missing():
    """
    Test to the inverse of service.available
    """
    mock = MagicMock(return_value=["c", "a", "b"])
    with patch.object(win_service, "get_all", mock):
        assert win_service.missing("d") is True


def test_get_all():
    """
    Test to return all installed services
    """
    mock = MagicMock(
        return_value=[
            {"ServiceName": "spongebob"},
            {"ServiceName": "squarepants"},
            {"ServiceName": "patrick"},
        ]
    )
    with patch.object(win_service, "_get_services", mock):
        expected = ["patrick", "spongebob", "squarepants"]
        result = win_service.get_all()
        assert result == expected


def test_get_service_name():
    """
    Test to the Display Name is what is displayed
    in Windows when services.msc is executed.
    """
    mock = MagicMock(
        return_value=[
            {"ServiceName": "spongebob", "DisplayName": "Sponge Bob"},
            {"ServiceName": "squarepants", "DisplayName": "Square Pants"},
            {"ServiceName": "patrick", "DisplayName": "Patrick the Starfish"},
        ]
    )
    with patch.object(win_service, "_get_services", mock):
        expected = {
            "Patrick the Starfish": "patrick",
            "Sponge Bob": "spongebob",
            "Square Pants": "squarepants",
        }
        result = win_service.get_service_name()
        assert result == expected

        expected = {"Patrick the Starfish": "patrick"}
        result = win_service.get_service_name("patrick")
        assert result == expected


@pytest.mark.skipif(not WINAPI, reason="win32serviceutil not available")
@pytest.mark.slow_test
def test_start():
    """
    Test to start the specified service
    """
    mock_true = MagicMock(return_value=True)
    mock_false = MagicMock(return_value=False)
    mock_info = MagicMock(side_effect=[{"Status": "Running"}])

    with patch.object(win32serviceutil, "StartService", mock_true), patch.object(
        win_service, "disabled", mock_false
    ), patch.object(win_service, "info", mock_info):
        assert win_service.start("spongebob") is True

    mock_info = MagicMock(
        side_effect=[
            {"Status": "Stopped", "Status_WaitHint": 0},
            {"Status": "Start Pending", "Status_WaitHint": 0},
            {"Status": "Running"},
        ]
    )

    with patch.object(win32serviceutil, "StartService", mock_true), patch.object(
        win_service, "disabled", mock_false
    ), patch.object(win_service, "info", mock_info), patch.object(
        win_service, "status", mock_true
    ):
        assert win_service.start("spongebob") is True


@pytest.mark.skipif(not WINAPI, reason="pywintypes not available")
def test_start_already_running():
    """
    Test starting a service that is already running
    """
    mock_false = MagicMock(return_value=False)
    mock_error = MagicMock(
        side_effect=pywintypes.error(1056, "StartService", "Service is running")
    )
    mock_info = MagicMock(side_effect=[{"Status": "Running"}])
    with patch.object(win32serviceutil, "StartService", mock_error), patch.object(
        win_service, "disabled", mock_false
    ), patch.object(win_service, "_status_wait", mock_info):
        assert win_service.start("spongebob") is True


@pytest.mark.skipif(not WINAPI, reason="win32serviceutil not available")
@pytest.mark.slow_test
def test_stop():
    """
    Test to stop the specified service
    """
    mock_true = MagicMock(return_value=True)
    mock_false = MagicMock(return_value=False)
    mock_info = MagicMock(side_effect=[{"Status": "Stopped"}])

    with patch.object(win32serviceutil, "StopService", mock_true), patch.object(
        win_service, "_status_wait", mock_info
    ):
        assert win_service.stop("spongebob") is True

    mock_info = MagicMock(
        side_effect=[
            {"Status": "Running", "Status_WaitHint": 0},
            {"Status": "Stop Pending", "Status_WaitHint": 0},
            {"Status": "Stopped"},
        ]
    )

    with patch.object(win32serviceutil, "StopService", mock_true), patch.object(
        win_service, "info", mock_info
    ), patch.object(win_service, "status", mock_false):
        assert win_service.stop("spongebob") is True


@pytest.mark.skipif(not WINAPI, reason="pywintypes not available")
def test_stop_not_running():
    """
    Test stopping a service that is already stopped
    """
    mock_error = MagicMock(
        side_effect=pywintypes.error(1062, "StopService", "Service is not running")
    )
    mock_info = MagicMock(side_effect=[{"Status": "Stopped"}])
    with patch.object(win32serviceutil, "StopService", mock_error), patch.object(
        win_service, "_status_wait", mock_info
    ):
        assert win_service.stop("spongebob") is True


def test_restart():
    """
    Test to restart the named service
    """
    mock_true = MagicMock(return_value=True)
    with patch.object(win_service, "create_win_salt_restart_task", mock_true):
        with patch.object(win_service, "execute_salt_restart_task", mock_true):
            assert win_service.restart("salt-minion") is True

    with patch.object(win_service, "stop", mock_true):
        with patch.object(win_service, "start", mock_true):
            assert win_service.restart("salt") is True


def test_createwin_saltrestart_task():
    """
    Test to create a task in Windows task
    scheduler to enable restarting the salt-minion
    """
    cmd = salt.utils.path.which("cmd")
    mock = MagicMock()
    with patch.dict(win_service.__salt__, {"task.create_task": mock}):
        win_service.create_win_salt_restart_task()
        mock.assert_called_once_with(
            action_type="Execute",
            arguments=(
                "/c ping -n 3 127.0.0.1 && net stop salt-minion && "
                "net start salt-minion"
            ),
            cmd=cmd,
            force=True,
            name="restart-salt-minion",
            start_date="1975-01-01",
            start_time="01:00",
            trigger_type="Once",
            user_name="System",
        )


def test_execute_salt_restart_task():
    """
    Test to run the Windows Salt restart task
    """
    mock_true = MagicMock(return_value=True)
    with patch.dict(win_service.__salt__, {"task.run": mock_true}):
        assert win_service.execute_salt_restart_task() is True


@pytest.mark.skipif(not WINAPI, reason="win32serviceutil not available")
def test_status():
    """
    Test to return the status for a service
    """
    mock_info = MagicMock(
        side_effect=[
            {"Status": "Running"},
            {"Status": "Stop Pending"},
            {"Status": "Stopped"},
            CommandExecutionError,
        ]
    )

    with patch.object(win_service, "info", mock_info):
        assert win_service.status("spongebob") is True
        assert win_service.status("patrick") is True
        assert win_service.status("squidward") is False
        assert win_service.status("non_existing") == "Not Found"


def test_getsid():
    """
    Test to return the sid for this windows service
    """
    mock_info = MagicMock(
        side_effect=[{"sid": "S-1-5-80-1956725871..."}, {"sid": None}]
    )
    with patch.object(win_service, "info", mock_info):
        expected = "S-1-5-80-1956725871..."
        result = win_service.getsid("spongebob")
        assert result == expected
        assert win_service.getsid("plankton") is None


def test_enable():
    """
    Test to enable the named service to start at boot
    """
    mock_modify = MagicMock(return_value=True)
    mock_info = MagicMock(return_value={"StartType": "Auto", "StartTypeDelayed": False})
    with patch.object(win_service, "modify", mock_modify):
        with patch.object(win_service, "info", mock_info):
            assert win_service.enable("spongebob") is True


def test_disable():
    """
    Test to disable the named service to start at boot
    """
    mock_modify = MagicMock(return_value=True)
    mock_info = MagicMock(return_value={"StartType": "Disabled"})
    with patch.object(win_service, "modify", mock_modify):
        with patch.object(win_service, "info", mock_info):
            assert win_service.disable("spongebob") is True


def test_enabled():
    """
    Test to check to see if the named
    service is enabled to start on boot
    """
    mock = MagicMock(side_effect=[{"StartType": "Auto"}, {"StartType": "Disabled"}])
    with patch.object(win_service, "info", mock):
        assert win_service.enabled("spongebob") is True
        assert win_service.enabled("squarepants") is False


def test_enabled_with_space_in_name():
    """
    Test to check to see if the named
    service is enabled to start on boot
    when have space in service name
    """
    mock = MagicMock(side_effect=[{"StartType": "Auto"}, {"StartType": "Disabled"}])
    with patch.object(win_service, "info", mock):
        assert win_service.enabled("spongebob test") is True
        assert win_service.enabled("squarepants test") is False


def test_disabled():
    """
    Test to check to see if the named
    service is disabled to start on boot
    """
    mock = MagicMock(side_effect=[False, True])
    with patch.object(win_service, "enabled", mock):
        assert win_service.disabled("spongebob") is True
        assert win_service.disabled("squarepants") is False


def test_cmd_quote():
    """
    Make sure the command gets quoted correctly
    """
    # Should always return command wrapped in double quotes
    expected = r'"C:\Program Files\salt\test.exe"'

    # test no quotes
    bin_path = r"C:\Program Files\salt\test.exe"
    result = win_service._cmd_quote(bin_path)
    assert result == expected

    # test single quotes
    bin_path = r"'C:\Program Files\salt\test.exe'"
    result = win_service._cmd_quote(bin_path)
    assert result == expected

    # test double quoted single quotes
    bin_path = "\"'C:\\Program Files\\salt\\test.exe'\""
    result = win_service._cmd_quote(bin_path)
    assert result == expected

    # test single quoted, double quoted, single quotes
    bin_path = "'\"'C:\\Program Files\\salt\\test.exe'\"'"
    result = win_service._cmd_quote(bin_path)
    assert result == expected



import pytest

import salt.config
import salt.daemons.masterapi as masterapi
import salt.utils.platform
from tests.support.mock import MagicMock, patch

pytestmark = [
    pytest.mark.slow_test,
]


@pytest.fixture
def check_keys():
    return {
        "test": "mGXdurU1c8lXt5cmpbGq4rWvrOvDXxkwI9gbkP5CBBjpyGWuB8vkgz9r+sjjG0wVDL9/uFuREtk=",
        "root": "2t5HHv/ek2wIFh8tTX2c3hdt+6V+93xKlcXb7IlGLIszOeCVv2NuH38LyCw9UwQTfUFTeseXhSs=",
    }


@pytest.fixture
def local_funcs(master_opts):
    opts = salt.config.master_config(None)
    return masterapi.LocalFuncs(opts, "test-key")


@pytest.fixture
def check_local_funcs(master_opts, check_keys):
    return masterapi.LocalFuncs(master_opts, check_keys)


# runner tests


def test_runner_token_not_authenticated(local_funcs):
    """
    Asserts that a TokenAuthenticationError is returned when the token can't authenticate.
    """
    mock_ret = {
        "error": {
            "name": "TokenAuthenticationError",
            "message": 'Authentication failure of type "token" occurred.',
        }
    }
    ret = local_funcs.runner({"token": "asdfasdfasdfasdf"})
    assert mock_ret == ret


def test_runner_token_authorization_error(local_funcs):
    """
    Asserts that a TokenAuthenticationError is returned when the token authenticates, but is
    not authorized.
    """
    token = "asdfasdfasdfasdf"
    load = {"token": token, "fun": "test.arg", "kwarg": {}}
    mock_token = {"token": token, "eauth": "foo", "name": "test"}
    mock_ret = {
        "error": {
            "name": "TokenAuthenticationError",
            "message": (
                'Authentication failure of type "token" occurred for user test.'
            ),
        }
    }

    with patch(
        "salt.auth.LoadAuth.authenticate_token", MagicMock(return_value=mock_token)
    ), patch("salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=[])):
        ret = local_funcs.runner(load)

    assert mock_ret == ret


def test_runner_token_salt_invocation_error(local_funcs):
    """
    Asserts that a SaltInvocationError is returned when the token authenticates, but the
    command is malformed.
    """
    token = "asdfasdfasdfasdf"
    load = {"token": token, "fun": "badtestarg", "kwarg": {}}
    mock_token = {"token": token, "eauth": "foo", "name": "test"}
    mock_ret = {
        "error": {
            "name": "SaltInvocationError",
            "message": "A command invocation error occurred: Check syntax.",
        }
    }

    with patch(
        "salt.auth.LoadAuth.authenticate_token", MagicMock(return_value=mock_token)
    ), patch("salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=["testing"])):
        ret = local_funcs.runner(load)

    assert mock_ret == ret


def test_runner_eauth_not_authenticated(local_funcs):
    """
    Asserts that an EauthAuthenticationError is returned when the user can't authenticate.
    """
    mock_ret = {
        "error": {
            "name": "EauthAuthenticationError",
            "message": (
                'Authentication failure of type "eauth" occurred for user UNKNOWN.'
            ),
        }
    }
    ret = local_funcs.runner({"eauth": "foo"})
    assert mock_ret == ret


def test_runner_eauth_authorization_error(local_funcs):
    """
    Asserts that an EauthAuthenticationError is returned when the user authenticates, but is
    not authorized.
    """
    load = {"eauth": "foo", "username": "test", "fun": "test.arg", "kwarg": {}}
    mock_ret = {
        "error": {
            "name": "EauthAuthenticationError",
            "message": (
                'Authentication failure of type "eauth" occurred for user test.'
            ),
        }
    }
    with patch(
        "salt.auth.LoadAuth.authenticate_eauth", MagicMock(return_value=True)
    ), patch("salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=[])):
        ret = local_funcs.runner(load)

    assert mock_ret == ret


def test_runner_eauth_salt_invocation_error(local_funcs):
    """
    Asserts that an EauthAuthenticationError is returned when the user authenticates, but the
    command is malformed.
    """
    load = {
        "eauth": "foo",
        "username": "test",
        "fun": "bad.test.arg.func",
        "kwarg": {},
    }
    mock_ret = {
        "error": {
            "name": "SaltInvocationError",
            "message": "A command invocation error occurred: Check syntax.",
        }
    }
    with patch(
        "salt.auth.LoadAuth.authenticate_eauth", MagicMock(return_value=True)
    ), patch("salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=["testing"])):
        ret = local_funcs.runner(load)

    assert mock_ret == ret


# wheel tests


def test_wheel_token_not_authenticated(local_funcs):
    """
    Asserts that a TokenAuthenticationError is returned when the token can't authenticate.
    """
    mock_ret = {
        "error": {
            "name": "TokenAuthenticationError",
            "message": 'Authentication failure of type "token" occurred.',
        }
    }
    ret = local_funcs.wheel({"token": "asdfasdfasdfasdf"})
    assert mock_ret == ret


def test_wheel_token_authorization_error(local_funcs):
    """
    Asserts that a TokenAuthenticationError is returned when the token authenticates, but is
    not authorized.
    """
    token = "asdfasdfasdfasdf"
    load = {"token": token, "fun": "test.arg", "kwarg": {}}
    mock_token = {"token": token, "eauth": "foo", "name": "test"}
    mock_ret = {
        "error": {
            "name": "TokenAuthenticationError",
            "message": (
                'Authentication failure of type "token" occurred for user test.'
            ),
        }
    }

    with patch(
        "salt.auth.LoadAuth.authenticate_token", MagicMock(return_value=mock_token)
    ), patch("salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=[])):
        ret = local_funcs.wheel(load)

    assert mock_ret == ret


def test_wheel_token_salt_invocation_error(local_funcs):
    """
    Asserts that a SaltInvocationError is returned when the token authenticates, but the
    command is malformed.
    """
    token = "asdfasdfasdfasdf"
    load = {"token": token, "fun": "badtestarg", "kwarg": {}}
    mock_token = {"token": token, "eauth": "foo", "name": "test"}
    mock_ret = {
        "error": {
            "name": "SaltInvocationError",
            "message": "A command invocation error occurred: Check syntax.",
        }
    }

    with patch(
        "salt.auth.LoadAuth.authenticate_token", MagicMock(return_value=mock_token)
    ), patch("salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=["testing"])):
        ret = local_funcs.wheel(load)

    assert mock_ret == ret


def test_wheel_eauth_not_authenticated(local_funcs):
    """
    Asserts that an EauthAuthenticationError is returned when the user can't authenticate.
    """
    mock_ret = {
        "error": {
            "name": "EauthAuthenticationError",
            "message": (
                'Authentication failure of type "eauth" occurred for user UNKNOWN.'
            ),
        }
    }
    ret = local_funcs.wheel({"eauth": "foo"})
    assert mock_ret == ret


def test_wheel_eauth_authorization_error(local_funcs):
    """
    Asserts that an EauthAuthenticationError is returned when the user authenticates, but is
    not authorized.
    """
    load = {"eauth": "foo", "username": "test", "fun": "test.arg", "kwarg": {}}
    mock_ret = {
        "error": {
            "name": "EauthAuthenticationError",
            "message": (
                'Authentication failure of type "eauth" occurred for user test.'
            ),
        }
    }
    with patch(
        "salt.auth.LoadAuth.authenticate_eauth", MagicMock(return_value=True)
    ), patch("salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=[])):
        ret = local_funcs.wheel(load)

    assert mock_ret == ret


def test_wheel_eauth_salt_invocation_error(local_funcs):
    """
    Asserts that an EauthAuthenticationError is returned when the user authenticates, but the
    command is malformed.
    """
    load = {
        "eauth": "foo",
        "username": "test",
        "fun": "bad.test.arg.func",
        "kwarg": {},
    }
    mock_ret = {
        "error": {
            "name": "SaltInvocationError",
            "message": "A command invocation error occurred: Check syntax.",
        }
    }
    with patch(
        "salt.auth.LoadAuth.authenticate_eauth", MagicMock(return_value=True)
    ), patch("salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=["testing"])):
        ret = local_funcs.wheel(load)

    assert mock_ret == ret


def test_wheel_user_not_authenticated(local_funcs):
    """
    Asserts that an UserAuthenticationError is returned when the user can't authenticate.
    """
    mock_ret = {
        "error": {
            "name": "UserAuthenticationError",
            "message": (
                'Authentication failure of type "user" occurred for user UNKNOWN.'
            ),
        }
    }
    ret = local_funcs.wheel({})
    assert mock_ret == ret


# publish tests


def test_publish_user_is_blacklisted(local_funcs):
    """
    Asserts that an AuthorizationError is returned when the user has been blacklisted.
    """
    mock_ret = {
        "error": {
            "name": "AuthorizationError",
            "message": "Authorization error occurred.",
        }
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=True)
    ):
        assert mock_ret == local_funcs.publish({"user": "foo", "fun": "test.arg"})


def test_publish_cmd_blacklisted(local_funcs):
    """
    Asserts that an AuthorizationError is returned when the command has been blacklisted.
    """
    mock_ret = {
        "error": {
            "name": "AuthorizationError",
            "message": "Authorization error occurred.",
        }
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch("salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=True)):
        assert mock_ret == local_funcs.publish({"user": "foo", "fun": "test.arg"})


def test_publish_token_not_authenticated(local_funcs):
    """
    Asserts that an AuthenticationError is returned when the token can't authenticate.
    """
    load = {
        "user": "foo",
        "fun": "test.arg",
        "tgt": "test_minion",
        "kwargs": {"token": "asdfasdfasdfasdf"},
    }
    mock_ret = {
        "error": {
            "name": "AuthenticationError",
            "message": "Authentication error occurred.",
        }
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch("salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)):
        assert mock_ret == local_funcs.publish(load)


def test_publish_token_authorization_error(local_funcs):
    """
    Asserts that an AuthorizationError is returned when the token authenticates, but is not
    authorized.
    """
    token = "asdfasdfasdfasdf"
    load = {
        "user": "foo",
        "fun": "test.arg",
        "tgt": "test_minion",
        "arg": "bar",
        "kwargs": {"token": token},
    }
    mock_token = {"token": token, "eauth": "foo", "name": "test"}
    mock_ret = {
        "error": {
            "name": "AuthorizationError",
            "message": "Authorization error occurred.",
        }
    }

    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.auth.LoadAuth.authenticate_token", MagicMock(return_value=mock_token)
    ), patch(
        "salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=[])
    ):
        assert mock_ret == local_funcs.publish(load)


def test_publish_eauth_not_authenticated(local_funcs):
    """
    Asserts that an AuthenticationError is returned when the user can't authenticate.
    """
    load = {
        "user": "test",
        "fun": "test.arg",
        "tgt": "test_minion",
        "kwargs": {"eauth": "foo"},
    }
    mock_ret = {
        "error": {
            "name": "AuthenticationError",
            "message": "Authentication error occurred.",
        }
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch("salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)):
        assert mock_ret == local_funcs.publish(load)


def test_publish_eauth_authorization_error(local_funcs):
    """
    Asserts that an AuthorizationError is returned when the user authenticates, but is not
    authorized.
    """
    load = {
        "user": "test",
        "fun": "test.arg",
        "tgt": "test_minion",
        "kwargs": {"eauth": "foo"},
        "arg": "bar",
    }
    mock_ret = {
        "error": {
            "name": "AuthorizationError",
            "message": "Authorization error occurred.",
        }
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.auth.LoadAuth.authenticate_eauth", MagicMock(return_value=True)
    ), patch(
        "salt.auth.LoadAuth.get_auth_list", MagicMock(return_value=[])
    ):
        assert mock_ret == local_funcs.publish(load)


def test_publish_user_not_authenticated(local_funcs):
    """
    Asserts that an AuthenticationError is returned when the user can't authenticate.
    """
    load = {"user": "test", "fun": "test.arg", "tgt": "test_minion"}
    mock_ret = {
        "error": {
            "name": "AuthenticationError",
            "message": "Authentication error occurred.",
        }
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch("salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)):
        assert mock_ret == local_funcs.publish(load)


def test_publish_user_authenticated_missing_auth_list(local_funcs):
    """
    Asserts that an AuthenticationError is returned when the user has an effective user id and is
    authenticated, but the auth_list is empty.
    """
    load = {
        "user": "test",
        "fun": "test.arg",
        "tgt": "test_minion",
        "kwargs": {"user": "test"},
        "arg": "foo",
    }
    mock_ret = {
        "error": {
            "name": "AuthenticationError",
            "message": "Authentication error occurred.",
        }
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.auth.LoadAuth.authenticate_key",
        MagicMock(return_value="fake-user-key"),
    ), patch(
        "salt.utils.master.get_values_of_matching_keys", MagicMock(return_value=[])
    ):
        assert mock_ret == local_funcs.publish(load)


def test_publish_user_authorization_error(local_funcs):
    """
    Asserts that an AuthorizationError is returned when the user authenticates, but is not
    authorized.
    """
    load = {
        "user": "test",
        "fun": "test.arg",
        "tgt": "test_minion",
        "kwargs": {"user": "test"},
        "arg": "foo",
    }
    mock_ret = {
        "error": {
            "name": "AuthorizationError",
            "message": "Authorization error occurred.",
        }
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.auth.LoadAuth.authenticate_key",
        MagicMock(return_value="fake-user-key"),
    ), patch(
        "salt.utils.master.get_values_of_matching_keys",
        MagicMock(return_value=["test"]),
    ), patch(
        "salt.utils.minions.CkMinions.auth_check", MagicMock(return_value=False)
    ):
        assert mock_ret == local_funcs.publish(load)


def test_dual_key_auth(check_local_funcs):
    """
    Test for check for presented dual keys (salt, root) are authenticated
    """
    load = {
        "user": "test",
        "fun": "test.arg",
        "tgt": "test_minion",
        "kwargs": {"user": "test"},
        "arg": "foo",
        "key": "mGXdurU1c8lXt5cmpbGq4rWvrOvDXxkwI9gbkP5CBBjpyGWuB8vkgz9r+sjjG0wVDL9/uFuREtk=",
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.utils.master.get_values_of_matching_keys",
        MagicMock(return_value=["test"]),
    ):
        results = check_local_funcs.publish(load)
        assert results == {"enc": "clear", "load": {"jid": None, "minions": []}}


def test_dual_key_auth_sudo(check_local_funcs):
    """
    Test for check for presented dual keys (salt, root) are authenticated
    with a sudo user
    """
    load = {
        "user": "sudo_test",
        "fun": "test.arg",
        "tgt": "test_minion",
        "kwargs": {"user": "sudo_test"},
        "arg": "foo",
        "key": "mGXdurU1c8lXt5cmpbGq4rWvrOvDXxkwI9gbkP5CBBjpyGWuB8vkgz9r+sjjG0wVDL9/uFuREtk=",
    }
    with patch(
        "salt.acl.PublisherACL.user_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.acl.PublisherACL.cmd_is_blacklisted", MagicMock(return_value=False)
    ), patch(
        "salt.utils.master.get_values_of_matching_keys",
        MagicMock(return_value=["test"]),
    ):
        results = check_local_funcs.publish(load)
        assert results == {"enc": "clear", "load": {"jid": None, "minions": []}}



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

import pytest

import salt.modules.sensors as sensors
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {sensors: {}}


def test_sense():
    """
    Test to gather lm-sensors data from a given chip
    """
    with patch.dict(
        sensors.__salt__, {"cmd.run": MagicMock(return_value="A:a B:b C:c D:d")}
    ):
        assert sensors.sense("chip") == {"A": "a B"}



import pytest
from salt.utils.templates import render_wempy_tmpl

pytest.importorskip("wemplate")


def test_render_wempy_sanity(render_context):
    tmpl = """OK"""
    res = render_wempy_tmpl(tmpl, render_context)
    assert res == "OK"


def test_render_wempy_evaluate(render_context):
    tmpl = """{{="OK"}}"""
    res = render_wempy_tmpl(tmpl, render_context)
    assert res == "OK"


def test_render_wempy_evaluate_multi(render_context):
    tmpl = """{{if 1:}}OK{{pass}}"""
    res = render_wempy_tmpl(tmpl, render_context)
    assert res == "OK"


def test_render_wempy_variable(render_context):
    tmpl = """{{=var}}"""
    render_context["var"] = "OK"
    res = render_wempy_tmpl(tmpl, render_context)
    assert res == "OK"



"""
DSON Renderer for Salt

This renderer is intended for demonstration purposes. Information on the DSON
spec can be found `here`__.

.. __: http://vpzomtrrfrt.github.io/DSON/

This renderer requires `Dogeon`__ (installable via pip)

.. __: https://github.com/soasme/dogeon
"""

import logging

try:
    import dson
except ImportError:
    dson = None


log = logging.getLogger(__name__)


def __virtual__():
    if dson is None:
        return (False, "The dogeon Python package is not installed")
    return True


def render(dson_input, saltenv="base", sls="", **kwargs):
    """
    Accepts DSON data as a string or as a file object and runs it through the
    JSON parser.

    :rtype: A Python data structure
    """
    if not isinstance(dson_input, str):
        dson_input = dson_input.read()

    log.debug("DSON input = %s", dson_input)

    if dson_input.startswith("#!"):
        dson_input = dson_input[(dson_input.find("\n") + 1) :]
    if not dson_input.strip():
        return {}
    return dson.loads(dson_input)



import random

import pytest

import salt.modules.cmdmod
import salt.utils.platform
import salt.utils.win_lgpo_auditpol as win_lgpo_auditpol
from tests.support.mock import MagicMock, patch

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
]


@pytest.fixture(scope="module")
def settings():
    return ["No Auditing", "Success", "Failure", "Success and Failure"]


@pytest.fixture
def configure_loader_modules():
    return {
        win_lgpo_auditpol: {
            "__context__": {},
            "__salt__": {"cmd.run_all": salt.modules.cmdmod.run_all},
        }
    }


def test_get_settings():
    names = win_lgpo_auditpol._get_valid_names()
    ret = win_lgpo_auditpol.get_settings(category="All")
    for name in names:
        assert name in [k.lower() for k in ret]


def test_get_settings_invalid_category():
    pytest.raises(KeyError, win_lgpo_auditpol.get_settings, category="Fake Category")


@pytest.mark.slow_test
def test_get_setting(settings):
    names = win_lgpo_auditpol._get_valid_names()
    for name in names:
        ret = win_lgpo_auditpol.get_setting(name)
        assert ret in settings


def test_get_setting_invalid_name():
    pytest.raises(KeyError, win_lgpo_auditpol.get_setting, name="Fake Name")


def test_set_setting(settings):
    names = ["Credential Validation", "IPsec Driver", "File System", "SAM"]
    mock_set = MagicMock(return_value={"retcode": 0, "stdout": "Success"})
    with patch.object(salt.modules.cmdmod, "run_all", mock_set):
        with patch.object(
            win_lgpo_auditpol,
            "_get_valid_names",
            return_value=[k.lower() for k in names],
        ):
            for name in names:
                value = random.choice(settings)
                win_lgpo_auditpol.set_setting(name=name, value=value)
                switches = win_lgpo_auditpol.settings[value]
                cmd = f'auditpol /set /subcategory:"{name}" {switches}'
                mock_set.assert_called_once_with(cmd=cmd, python_shell=True)
                mock_set.reset_mock()


def test_set_setting_invalid_setting():
    names = ["Credential Validation", "IPsec Driver", "File System"]
    with patch.object(
        win_lgpo_auditpol,
        "_get_valid_names",
        return_value=[k.lower() for k in names],
    ):
        pytest.raises(
            KeyError,
            win_lgpo_auditpol.set_setting,
            name="Fake Name",
            value="No Auditing",
        )


def test_set_setting_invalid_value():
    names = ["Credential Validation", "IPsec Driver", "File System"]
    with patch.object(
        win_lgpo_auditpol,
        "_get_valid_names",
        return_value=[k.lower() for k in names],
    ):
        pytest.raises(
            KeyError,
            win_lgpo_auditpol.set_setting,
            name="Credential Validation",
            value="Fake Value",
        )


def test_get_auditpol_dump():
    names = win_lgpo_auditpol._get_valid_names()
    dump = win_lgpo_auditpol.get_auditpol_dump()
    for name in names:
        found = False
        for line in dump:
            if name.lower() in line.lower():
                found = True
                break
        assert found is True



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>

    Test cases for salt.modules.solr
"""

import os

import pytest

import salt.modules.solr as solr
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {solr: {}}


def test_lucene_version():
    """
    Test to get the lucene version that solr is using.
    """
    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.object(solr, "_get_none_or_value", side_effect=[None, True, True]):
            with patch.object(
                solr, "_check_for_cores", side_effect=[True, False, False]
            ):
                tempdict = {
                    "success": "success",
                    "errors": "errors",
                    "data": {"lucene": {"lucene-spec-version": 1}},
                }
                with patch.object(
                    solr,
                    "_get_admin_info",
                    side_effect=[tempdict, tempdict, {"success": None}],
                ):
                    with patch.dict(
                        solr.__salt__,
                        {"config.option": MagicMock(return_value=["A"])},
                    ):

                        with patch.object(
                            solr, "_update_return_dict", return_value={"A": "a"}
                        ):
                            assert solr.lucene_version("c") == {"A": "a"}

                        assert solr.lucene_version("c") == {"A": "a"}

                        assert solr.lucene_version("c") == {"success": None}


def test_version():
    """
    Test to get the solr version for the core specified
    """
    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.object(solr, "_get_none_or_value", side_effect=[None, True, True]):
            with patch.object(
                solr, "_check_for_cores", side_effect=[True, False, False]
            ):

                tempdict = {
                    "success": "success",
                    "errors": "errors",
                    "warnings": "warnings",
                    "data": {"lucene": {"solr-spec-version": 1}},
                }
                with patch.object(
                    solr, "_get_admin_info", side_effect=[tempdict, tempdict]
                ):
                    with patch.dict(
                        solr.__opts__, {"solr.cores": MagicMock(return_value=["A"])}
                    ):
                        with patch.object(
                            solr, "_update_return_dict", return_value={"A": "a"}
                        ):
                            assert solr.version() == {"A": "a"}

                        assert solr.version() == {"A": "a"}

                with patch.object(
                    solr, "_get_admin_info", return_value={"success": None}
                ):
                    assert solr.version() == {"success": None}


def test_optimize():
    """
    Test to search queries fast, but it is a very expensive operation.
    """
    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.object(solr, "_get_none_or_value", side_effect=[None, True]):
            with patch.object(solr, "_check_for_cores", side_effect=[True, False]):
                tempdict = {
                    "success": "success",
                    "errors": "errors",
                    "warnings": "warnings",
                    "data": {"lucene": {"solr-spec-version": 1}},
                }
                with patch.object(solr, "_format_url", return_value="A"):
                    with patch.dict(
                        solr.__salt__,
                        {"config.option": MagicMock(return_value=["A"])},
                    ):
                        with patch.object(solr, "_http_request", return_value=tempdict):
                            with patch.object(
                                solr, "_update_return_dict", return_value={"A": "a"}
                            ):
                                assert solr.optimize() == {"A": "a"}

                    with patch.object(solr, "_http_request", return_value="A"):
                        assert solr.optimize() == "A"


def test_ping():
    """
    Test to check on solr, makes sure solr can talk to the
    indexes.
    """
    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.object(solr, "_get_none_or_value", side_effect=[None, True]):
            with patch.object(solr, "_check_for_cores", side_effect=[True, False]):

                tempdict = {
                    "success": "success",
                    "errors": "errors",
                    "warnings": "warnings",
                    "data": {"lucene": {"solr-spec-version": 1}},
                }

                with patch.dict(
                    solr.__opts__, {"solr.cores": MagicMock(return_value=["A"])}
                ):
                    with patch.object(solr, "_get_admin_info", return_value=tempdict):
                        with patch.object(
                            solr, "_update_return_dict", return_value={"A": "a"}
                        ):
                            assert solr.ping() == {"A": "a"}

                with patch.object(solr, "_get_admin_info", return_value="A"):
                    assert solr.ping() == "A"


def test_is_replication_enabled():
    """
    Test to check for errors, and determine if a slave
    is replicating or not.
    """
    error = 'Only "slave" minions can run "is_replication_enabled"'
    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.object(solr, "_is_master", side_effect=[True, False]):
            assert solr.is_replication_enabled() is None
            with patch.object(solr, "_get_none_or_value", return_value=None):
                with patch.object(solr, "_check_for_cores", return_value=True):
                    with patch.dict(
                        solr.__opts__, {"solr.cores": MagicMock(return_value="A")}
                    ):
                        with patch.object(
                            solr, "_replication_request", return_value="A"
                        ):
                            assert solr.is_replication_enabled() == {
                                "A": "a",
                                "errors": [error],
                                "success": False,
                            }


def test_match_index_versions():
    """
    Test to verifies that the master and the slave versions are in sync by
    comparing the index version.
    """
    err = 'solr.match_index_versions can only be called by "slave" minions'
    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.object(solr, "_is_master", side_effect=[True, False]):

            assert solr.match_index_versions() is None

            with patch.object(solr, "_get_none_or_value", return_value=None):
                with patch.object(solr, "_check_for_cores", return_value=True):
                    with patch.dict(
                        solr.__opts__, {"solr.cores": MagicMock(return_value="A")}
                    ):
                        with patch.object(
                            solr, "_replication_request", return_value="A"
                        ):
                            assert solr.match_index_versions() == {
                                "A": "a",
                                "errors": [err],
                                "success": False,
                            }


def test_replication_details():
    """
    Test to get the full replication details.
    """
    tempdict1 = {
        "success": "success",
        "errors": "errors",
        "warnings": "warnings",
        "data": "data",
    }

    tempdict2 = {
        "success": None,
        "errors": "errors",
        "warnings": "warnings",
        "data": "data",
    }

    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.object(solr, "_get_none_or_value", return_value=True):
            with patch.object(
                solr, "_replication_request", side_effect=[tempdict2, tempdict1]
            ):

                assert solr.replication_details() == tempdict2

                with patch.object(solr, "_update_return_dict", return_value=tempdict1):
                    assert solr.replication_details() == tempdict1


def test_backup():
    """
    Test to tell solr make a backup.
    """
    tempdict = {
        "success": "success",
        "errors": "errors",
        "warnings": "warnings",
        "data": "data",
    }

    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.dict(
            solr.__opts__,
            {
                "solr.backup_path": MagicMock(return_value="A"),
                "solr.num_backups": MagicMock(return_value="B"),
                "solr.cores": MagicMock(return_value=["A"]),
            },
        ):
            with patch.object(os.path, "sep", return_value="B"):
                with patch.object(solr, "_get_none_or_value", side_effect=[None, True]):
                    with patch.object(
                        solr, "_check_for_cores", side_effect=[True, False]
                    ):
                        with patch.object(
                            solr, "_replication_request", return_value=tempdict
                        ):
                            with patch.dict(
                                solr.__opts__,
                                {"solr.cores": MagicMock(return_value=["A"])},
                            ):
                                with patch.object(
                                    solr, "_update_return_dict", return_value="A"
                                ):
                                    assert solr.backup() == {"A": "a"}

                            assert solr.backup() == tempdict


def test_set_is_polling():
    """
    Test to prevent the slaves from polling the master for updates.
    """
    tempdict = {
        "success": "success",
        "errors": "errors",
        "warnings": "warnings",
        "data": "data",
    }

    err = 'solr.set_is_polling can only be called by "slave" minions'

    with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
        with patch.object(solr, "_is_master", side_effect=[True, False, False]):
            with patch.object(
                solr, "_get_none_or_value", side_effect=[None, None, True]
            ):
                with patch.object(solr, "_check_for_cores", side_effect=[True, False]):

                    assert solr.set_is_polling("p") is None

                    with patch.dict(
                        solr.__opts__, {"solr.cores": MagicMock(return_value="A")}
                    ):
                        with patch.object(
                            solr, "_update_return_dict", return_value=tempdict
                        ):
                            assert solr.set_is_polling("p") == {
                                "A": "a",
                                "errors": [err],
                                "success": False,
                            }

                    with patch.object(solr, "_replication_request", return_value="A"):
                        assert solr.set_is_polling("p") == "A"


def test_set_replication_enabled():
    """
    Test to sets the master to ignore poll requests from the slaves.
    """
    with patch.object(solr, "_is_master", side_effect=[False, True, True, True]):
        with patch.object(
            solr, "_get_none_or_value", side_effect=[None, None, True, True, True]
        ):
            with patch.object(solr, "_get_return_dict", side_effect=[{"A": "a"}, {}]):
                with patch.object(solr, "_replication_request", return_value="A"):
                    assert solr.set_replication_enabled("s") == {"A": "a"}
                    with patch.object(solr, "_check_for_cores", return_value=True):
                        with patch.dict(
                            solr.__opts__,
                            {"solr.cores": MagicMock(return_value="n")},
                        ):
                            assert solr.set_replication_enabled("s") == {}
                    assert solr.set_replication_enabled("s") == "A"
                    assert solr.set_replication_enabled(False) == "A"


def test_signal():
    """
    Test to signals Apache Solr to start, stop, or restart.
    """
    assert (
        solr.signal("signal")
        == "signal is an invalid signal. Try: one of: start, stop or restart"
    )


def test_reload_core():
    """
    Test to load a new core from the same configuration as
    an existing registered core.
    """
    error = ['solr.reload_core can only be called by "multi-core" minions']
    with patch.object(solr, "_check_for_cores", side_effect=[False, True, True, True]):
        with patch.object(solr, "_get_none_or_value", side_effect=[None, True]):
            with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
                with patch.object(solr, "_format_url", return_value="A"):
                    with patch.object(solr, "_http_request", return_value="A"):
                        with patch.dict(
                            solr.__opts__,
                            {"solr.cores": MagicMock(return_value="n")},
                        ):
                            assert solr.reload_core() is None
                            assert solr.reload_core() == {
                                "A": "a",
                                "errors": error,
                                "success": False,
                            }
                            assert solr.reload_core() == "A"


def test_core_status():
    """
    Test to get the status for a given core or all cores
    if no core is specified
    """
    error = ['solr.reload_core can only be called by "multi-core" minions']
    with patch.object(solr, "_check_for_cores", side_effect=[False, True, True, True]):
        with patch.object(solr, "_get_none_or_value", side_effect=[None, True]):
            with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
                with patch.object(solr, "_format_url", return_value="A"):
                    with patch.object(solr, "_http_request", return_value="A"):
                        with patch.dict(
                            solr.__opts__,
                            {"solr.cores": MagicMock(return_value="n")},
                        ):
                            assert solr.core_status() is None
                            assert solr.core_status() == {
                                "A": "a",
                                "errors": error,
                                "success": False,
                            }

                            assert solr.core_status() == "A"


def test_reload_import_config():
    """
    Test to re-loads the handler config XML file.
    """
    with patch.object(solr, "_is_master", side_effect=[False, True, True]):
        with patch.object(
            solr, "_get_none_or_value", side_effect=[None, None, None, True, True]
        ):
            with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
                with patch.object(solr, "_check_for_cores", side_effect=[True, False]):
                    with patch.object(solr, "_format_url", return_value="A"):
                        with patch.object(solr, "_http_request", return_value="A"):
                            assert solr.reload_import_config("h") == {"A": "a"}
                            assert solr.reload_import_config("h") == {"A": "a"}
                            assert solr.reload_import_config("h") == "A"


def test_abort_import():
    """
    Test to aborts an existing import command to the specified handler.
    """
    with patch.object(solr, "_is_master", side_effect=[False, True, True]):
        with patch.object(
            solr, "_get_none_or_value", side_effect=[None, None, None, True, True]
        ):
            with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
                with patch.object(solr, "_check_for_cores", side_effect=[True, False]):
                    with patch.object(solr, "_format_url", return_value="A"):
                        with patch.object(solr, "_http_request", return_value="A"):
                            assert solr.abort_import("h") == {"A": "a"}
                            assert solr.abort_import("h") == {"A": "a"}
                            assert solr.abort_import("h") == "A"


def test_full_import():
    """
    Test to submits an import command to the specified handler using
    specified options.
    """
    with patch(
        "salt.modules.solr._format_url", MagicMock(return_value="A")
    ), patch.object(
        solr, "_is_master", side_effect=[False, True, True, True, True, True]
    ), patch.object(
        solr, "_get_return_dict", return_value={"A": "a"}
    ), patch.object(
        solr, "_get_none_or_value", side_effect=[None, True, True, True, True]
    ), patch.object(
        solr, "_check_for_cores", side_effect=[True, False, False, False, False]
    ), patch.object(
        solr,
        "_pre_index_check",
        side_effect=[{"success": False}, {"success": True}, {"success": True}],
    ), patch.object(
        solr, "_merge_options", side_effect=[{"clean": True}, {"clean": False}]
    ), patch.object(
        solr, "set_replication_enabled", return_value={"success": False}
    ), patch.object(
        solr, "_http_request", return_value="A"
    ):

        assert solr.full_import("h") == {"A": "a"}
        assert solr.full_import("h") == {"A": "a"}
        assert solr.full_import("h") == {"success": False}
        assert solr.full_import("h") == {"A": "a"}
        assert solr.full_import("h") == "A"


def test_delta_import():
    """
    Test to submits an import command to the specified handler using
    specified options.
    """
    with patch(
        "salt.modules.solr._format_url", MagicMock(return_value="A")
    ), patch.object(
        solr, "_is_master", side_effect=[False, True, True, True, True]
    ), patch.object(
        solr, "_get_none_or_value", side_effect=[None, True, True, True, True]
    ), patch.object(
        solr, "_get_return_dict", return_value={"A": "a"}
    ), patch.object(
        solr,
        "_pre_index_check",
        side_effect=[
            {"success": False},
            {"success": True},
            {"success": True},
            {"success": True},
        ],
    ), patch.object(
        solr, "_merge_options", side_effect=[{"clean": True}, {"clean": False}]
    ), patch.object(
        solr, "_check_for_cores", side_effect=[True, False]
    ), patch.object(
        solr, "set_replication_enabled", return_value={"success": False}
    ), patch.object(
        solr, "_http_request", return_value="A"
    ):

        assert solr.delta_import("h") == {"A": "a"}
        assert solr.delta_import("h") == {"success": False}
        assert solr.delta_import("h") == {"A": "a"}
        assert solr.delta_import("h") == "A"


def test_import_status():
    """
    Test to submits an import command to the specified handler using
    specified options.
    """
    with patch.object(solr, "_is_master", side_effect=[False, True]):
        with patch.object(solr, "_get_none_or_value", side_effect=[None, True]):
            with patch.object(solr, "_get_return_dict", return_value={"A": "a"}):
                with patch.object(solr, "_format_url", return_value="A"):
                    with patch.object(solr, "_http_request", return_value="A"):
                        assert solr.import_status("h") == {"A": "a"}
                        assert solr.import_status("h") == "A"



"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

import pytest

import salt.modules.bluez_bluetooth as bluez
import salt.utils.validate.net
from salt.exceptions import CommandExecutionError
from tests.support.mock import MagicMock, patch


class MockBluetooth:
    """
    Mock class for bluetooth
    """

    def __init__(self):
        pass

    @staticmethod
    def discover_devices(lookup_names):
        """
        Mock method to return all Discoverable devices
        """
        return [["a", "b", "c"], ["d", "e", "f"]]


@pytest.fixture
def configure_loader_modules():
    return {bluez: {"bluetooth": MockBluetooth()}}


def test_version():
    """
    Test if return bluetooth version
    """
    mock = MagicMock(return_value="5.7")
    with patch.dict(bluez.__salt__, {"cmd.run": mock}):
        assert bluez.version() == {
            "PyBluez": "<= 0.18 (Unknown, but installed)",
            "Bluez": "5.7",
        }


def test_address_():
    """
    Test of getting address of bluetooth adapter
    """
    mock = MagicMock(return_value="hci : hci0")
    with patch.dict(bluez.__salt__, {"cmd.run": mock}):
        assert bluez.address_() == {
            "hci ": {"device": "hci ", "path": "/sys/class/bluetooth/hci "}
        }


def test_power():
    """
    Test of getting address of bluetooth adapter
    """
    mock = MagicMock(return_value={})
    with patch.object(bluez, "address_", mock):
        pytest.raises(CommandExecutionError, bluez.power, "hci0", "on")

    mock = MagicMock(return_value={"hci0": {"device": "hci0", "power": "on"}})
    with patch.object(bluez, "address_", mock):
        mock = MagicMock(return_value="")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert bluez.power("hci0", "on")

    mock = MagicMock(return_value={"hci0": {"device": "hci0", "power": "on"}})
    with patch.object(bluez, "address_", mock):
        mock = MagicMock(return_value="")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert not bluez.power("hci0", "off")


def test_discoverable():
    """
    Test of enabling bluetooth device
    """
    mock = MagicMock(
        side_effect=[
            {},
            {"hci0": {"device": "hci0", "power": "on"}},
            {"hci0": {"device": "hci0", "power": "on"}},
        ]
    )
    with patch.object(bluez, "address_", mock):
        pytest.raises(CommandExecutionError, bluez.discoverable, "hci0")

        mock = MagicMock(return_value="UP RUNNING ISCAN")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert bluez.discoverable("hci0")

        mock = MagicMock(return_value="")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert not bluez.discoverable("hci0")


def test_noscan():
    """
    Test of turning off of scanning modes
    """
    mock = MagicMock(
        side_effect=[
            {},
            {"hci0": {"device": "hci0", "power": "on"}},
            {"hci0": {"device": "hci0", "power": "on"}},
        ]
    )
    with patch.object(bluez, "address_", mock):
        pytest.raises(CommandExecutionError, bluez.noscan, "hci0")

        mock = MagicMock(return_value="SCAN")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert not bluez.noscan("hci0")

        mock = MagicMock(return_value="")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert bluez.noscan("hci0")


def test_scan():
    """
    Test of scanning of bluetooth devices
    """
    assert bluez.scan() == [{"a": "b"}, {"d": "e"}]


def test_block():
    """
    Test of blocking specific bluetooth device
    """
    mock = MagicMock(side_effect=[False, True])
    with patch.object(salt.utils.validate.net, "mac", mock):
        pytest.raises(CommandExecutionError, bluez.block, "DE:AD:BE:EF:CA:ZE")

        mock = MagicMock(return_value="")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert bluez.block("DE:AD:BE:EF:CA:FE") is None


def test_unblock():
    """
    Test to unblock specific bluetooth device
    """
    mock = MagicMock(side_effect=[False, True])
    with patch.object(salt.utils.validate.net, "mac", mock):
        pytest.raises(CommandExecutionError, bluez.block, "DE:AD:BE:EF:CA:ZE")

        mock = MagicMock(return_value="")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert bluez.unblock("DE:AD:BE:EF:CA:FE") is None


def test_pair():
    """
    Test to pair bluetooth adapter with a device
    """
    mock = MagicMock(side_effect=[False, True, True])
    with patch.object(salt.utils.validate.net, "mac", mock):
        pytest.raises(CommandExecutionError, bluez.pair, "DE:AD:BE:EF:CA:FE", "1234")

        pytest.raises(CommandExecutionError, bluez.pair, "DE:AD:BE:EF:CA:FE", "abcd")

        mock = MagicMock(return_value={"device": "hci0"})
        with patch.object(bluez, "address_", mock):
            mock = MagicMock(return_value="Ok")
            with patch.dict(bluez.__salt__, {"cmd.run": mock}):
                assert bluez.pair("DE:AD:BE:EF:CA:FE", "1234") == ["Ok"]


def test_unpair():
    """
    Test to unpair bluetooth adaptor with a device
    """
    mock = MagicMock(side_effect=[False, True])
    with patch.object(salt.utils.validate.net, "mac", mock):
        pytest.raises(CommandExecutionError, bluez.unpair, "DE:AD:BE:EF:CA:FE")

        mock = MagicMock(return_value="Ok")
        with patch.dict(bluez.__salt__, {"cmd.run": mock}):
            assert bluez.unpair("DE:AD:BE:EF:CA:FE") == ["Ok"]


def test_start():
    """
    Test to start bluetooth service
    """
    mock = MagicMock(return_value="Ok")
    with patch.dict(bluez.__salt__, {"service.start": mock}):
        assert bluez.start() == "Ok"


def test_stop():
    """
    Test to stop bluetooth service
    """
    mock = MagicMock(return_value="Ok")
    with patch.dict(bluez.__salt__, {"service.stop": mock}):
        assert bluez.stop() == "Ok"



import pytest


@pytest.mark.timeout_unless_on_windows(360)
def test_job_return(salt_master_1, salt_master_2, salt_minion_1):
    cli = salt_master_1.salt_cli(timeout=120)
    ret = cli.run("test.ping", "-v", minion_tgt=salt_minion_1.id)
    for line in ret.stdout.splitlines():
        if "with jid" in line:
            jid = line.split("with jid")[1].strip()

    run_1 = salt_master_1.salt_run_cli(timeout=120)
    ret = run_1.run("jobs.lookup_jid", jid)
    assert ret.data == {"minion-1": True}

    run_2 = salt_master_2.salt_run_cli(timeout=120)
    ret = run_2.run("jobs.lookup_jid", jid)
    assert ret.data == {}



"""
Unit tests for the Vault runner
"""

import logging

import pytest

import salt.runners.vault as vault
from tests.support.mock import ANY, MagicMock, Mock, patch

log = logging.getLogger(__name__)


def _mock_json_response(data, status_code=200, reason=""):
    """
    Mock helper for http response
    """
    response = MagicMock()
    response.json = MagicMock(return_value=data)
    response.status_code = status_code
    response.reason = reason
    return Mock(return_value=response)


@pytest.fixture
def configure_loader_modules():
    sig_valid_mock = patch(
        "salt.runners.vault._validate_signature", MagicMock(return_value=None)
    )
    token_url_mock = patch(
        "salt.runners.vault._get_token_create_url",
        MagicMock(return_value="http://fake_url"),
    )
    cached_policies = patch(
        "salt.runners.vault._get_policies_cached",
        Mock(return_value=["saltstack/minion/test-minion", "saltstack/minions"]),
    )
    with sig_valid_mock, token_url_mock, cached_policies:
        yield {
            vault: {
                "__opts__": {
                    "vault": {
                        "url": "http://127.0.0.1",
                        "auth": {
                            "token": "test",
                            "method": "token",
                            "allow_minion_override": True,
                        },
                    }
                }
            }
        }


def test_generate_token():
    """
    Basic tests for test_generate_token: all exits
    """
    mock = _mock_json_response(
        {"auth": {"client_token": "test", "renewable": False, "lease_duration": 0}}
    )
    with patch("requests.post", mock):
        result = vault.generate_token("test-minion", "signature")
        log.debug("generate_token result: %s", result)
        assert isinstance(result, dict)
        assert "error" not in result
        assert "token" in result
        assert result["token"] == "test"
        mock.assert_called_with(
            "http://fake_url", headers=ANY, json=ANY, verify=ANY, timeout=120
        )

        # Test uses
        num_uses = 6
        result = vault.generate_token("test-minion", "signature", uses=num_uses)
        assert "uses" in result
        assert result["uses"] == num_uses
        json_request = {
            "policies": ["saltstack/minion/test-minion", "saltstack/minions"],
            "num_uses": num_uses,
            "meta": {
                "saltstack-jid": "<no jid set>",
                "saltstack-minion": "test-minion",
                "saltstack-user": "<no user set>",
            },
        }
        mock.assert_called_with(
            "http://fake_url",
            headers=ANY,
            json=json_request,
            verify=ANY,
            timeout=120,
        )

        # Test ttl
        expected_ttl = "6h"
        result = vault.generate_token("test-minion", "signature", ttl=expected_ttl)
        assert result["uses"] == 1
        json_request = {
            "policies": ["saltstack/minion/test-minion", "saltstack/minions"],
            "num_uses": 1,
            "explicit_max_ttl": expected_ttl,
            "meta": {
                "saltstack-jid": "<no jid set>",
                "saltstack-minion": "test-minion",
                "saltstack-user": "<no user set>",
            },
        }
        mock.assert_called_with(
            "http://fake_url", headers=ANY, json=json_request, verify=ANY, timeout=120
        )

    mock = _mock_json_response({}, status_code=403, reason="no reason")
    with patch("requests.post", mock):
        result = vault.generate_token("test-minion", "signature")
        assert isinstance(result, dict)
        assert "error" in result
        assert result["error"] == "no reason"

    with patch("salt.runners.vault._get_policies_cached", MagicMock(return_value=[])):
        result = vault.generate_token("test-minion", "signature")
        assert isinstance(result, dict)
        assert "error" in result
        assert result["error"] == "No policies matched minion"

    with patch(
        "requests.post", MagicMock(side_effect=Exception("Test Exception Reason"))
    ):
        result = vault.generate_token("test-minion", "signature")
        assert isinstance(result, dict)
        assert "error" in result
        assert result["error"] == "Test Exception Reason"


def test_generate_token_with_namespace():
    """
    Basic tests for test_generate_token: all exits
    """
    mock = _mock_json_response(
        {"auth": {"client_token": "test", "renewable": False, "lease_duration": 0}}
    )
    supplied_config = {"namespace": "test_namespace"}
    with patch("requests.post", mock):
        with patch.dict(vault.__opts__["vault"], supplied_config):
            result = vault.generate_token("test-minion", "signature")
            log.debug("generate_token result: %s", result)
            assert isinstance(result, dict)
            assert "error" not in result
            assert "token" in result
            assert result["token"] == "test"
            mock.assert_called_with(
                "http://fake_url",
                headers={
                    "X-Vault-Token": "test",
                    "X-Vault-Namespace": "test_namespace",
                },
                json=ANY,
                verify=ANY,
                timeout=120,
            )



"""
:codeauthor: Shane Lee <slee@saltstack.com>
"""

import pytest

import salt.modules.win_lgpo as win_lgpo
from tests.support.mock import MagicMock, Mock, patch

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
    pytest.mark.slow_test,
]


def test__getAdmlDisplayName():
    display_name = "$(string.KeepAliveTime1)"
    adml_xml_data = "junk, we are mocking the return"
    obj_xpath = Mock()
    obj_xpath.text = "300000 or 5 minutes (recommended) "
    mock_xpath_obj = MagicMock(return_value=[obj_xpath])
    with patch.object(win_lgpo, "ADML_DISPLAY_NAME_XPATH", mock_xpath_obj):
        result = win_lgpo._getAdmlDisplayName(
            adml_xml_data=adml_xml_data, display_name=display_name
        )
    expected = "300000 or 5 minutes (recommended)"
    assert result == expected


def test__regexSearchKeyValueCombo_enabled():
    """
    Make sure
    """
    policy_data = (
        b"[\x00s\x00o\x00f\x00t\x00w\x00a\x00r\x00e\x00\\\x00p"
        b"\x00o\x00l\x00i\x00c\x00i\x00e\x00s\x00\\\x00m\x00i"
        b"\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00\\\x00w\x00i"
        b"\x00n\x00d\x00o\x00w\x00s\x00\\\x00w\x00i\x00n\x00d"
        b"\x00o\x00w\x00s\x00 \x00e\x00r\x00r\x00o\x00r\x00 "
        b"\x00r\x00e\x00p\x00o\x00r\x00t\x00i\x00n\x00g\x00\\"
        b"\x00c\x00o\x00n\x00s\x00e\x00n\x00t\x00\x00\x00;\x00D"
        b"\x00e\x00f\x00a\x00u\x00l\x00t\x00C\x00o\x00n\x00s"
        b"\x00e\x00n\x00t\x00\x00\x00;\x00\x01\x00\x00\x00;\x00"
        b"\x04\x00\x00\x00;\x00\x02\x00\x00\x00]\x00"
    )
    policy_regpath = (
        b"\x00s\x00o\x00f\x00t\x00w\x00a\x00r\x00e\x00\\\x00p"
        b"\x00o\x00l\x00i\x00c\x00i\x00e\x00s\x00\\\x00m\x00i"
        b"\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00\\\x00w\x00i"
        b"\x00n\x00d\x00o\x00w\x00s\x00\\\x00w\x00i\x00n\x00d"
        b"\x00o\x00w\x00s\x00 \x00e\x00r\x00r\x00o\x00r\x00 "
        b"\x00r\x00e\x00p\x00o\x00r\x00t\x00i\x00n\x00g\x00\\"
        b"\x00c\x00o\x00n\x00s\x00e\x00n\x00t\x00\x00"
    )
    policy_regkey = (
        b"\x00D\x00e\x00f\x00a\x00u\x00l\x00t\x00C\x00o\x00n"
        b"\x00s\x00e\x00n\x00t\x00\x00"
    )
    test = win_lgpo._regexSearchKeyValueCombo(
        policy_data=policy_data,
        policy_regpath=policy_regpath,
        policy_regkey=policy_regkey,
    )
    assert test == policy_data


def test__regexSearchKeyValueCombo_not_configured():
    """
    Make sure
    """
    policy_data = b""
    policy_regpath = (
        b"\x00s\x00o\x00f\x00t\x00w\x00a\x00r\x00e\x00\\\x00p"
        b"\x00o\x00l\x00i\x00c\x00i\x00e\x00s\x00\\\x00m\x00i"
        b"\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00\\\x00w\x00i"
        b"\x00n\x00d\x00o\x00w\x00s\x00\\\x00w\x00i\x00n\x00d"
        b"\x00o\x00w\x00s\x00 \x00e\x00r\x00r\x00o\x00r\x00 "
        b"\x00r\x00e\x00p\x00o\x00r\x00t\x00i\x00n\x00g\x00\\"
        b"\x00c\x00o\x00n\x00s\x00e\x00n\x00t\x00\x00"
    )
    policy_regkey = (
        b"\x00D\x00e\x00f\x00a\x00u\x00l\x00t\x00C\x00o\x00n"
        b"\x00s\x00e\x00n\x00t\x00\x00"
    )
    test = win_lgpo._regexSearchKeyValueCombo(
        policy_data=policy_data,
        policy_regpath=policy_regpath,
        policy_regkey=policy_regkey,
    )
    assert test is None


def test__regexSearchKeyValueCombo_disabled():
    """
    Make sure
    """
    policy_data = (
        b"[\x00s\x00o\x00f\x00t\x00w\x00a\x00r\x00e\x00\\\x00p"
        b"\x00o\x00l\x00i\x00c\x00i\x00e\x00s\x00\\\x00m\x00i"
        b"\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00\\\x00w\x00i"
        b"\x00n\x00d\x00o\x00w\x00s\x00\\\x00w\x00i\x00n\x00d"
        b"\x00o\x00w\x00s\x00 \x00e\x00r\x00r\x00o\x00r\x00 "
        b"\x00r\x00e\x00p\x00o\x00r\x00t\x00i\x00n\x00g\x00\\"
        b"\x00c\x00o\x00n\x00s\x00e\x00n\x00t\x00\x00\x00;\x00*"
        b"\x00*\x00d\x00e\x00l\x00.\x00D\x00e\x00f\x00a\x00u"
        b"\x00l\x00t\x00C\x00o\x00n\x00s\x00e\x00n\x00t\x00\x00"
        b"\x00;\x00\x01\x00\x00\x00;\x00\x04\x00\x00\x00;\x00 "
        b"\x00\x00\x00]\x00"
    )
    policy_regpath = (
        b"\x00s\x00o\x00f\x00t\x00w\x00a\x00r\x00e\x00\\\x00p"
        b"\x00o\x00l\x00i\x00c\x00i\x00e\x00s\x00\\\x00m\x00i"
        b"\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00\\\x00w\x00i"
        b"\x00n\x00d\x00o\x00w\x00s\x00\\\x00w\x00i\x00n\x00d"
        b"\x00o\x00w\x00s\x00 \x00e\x00r\x00r\x00o\x00r\x00 "
        b"\x00r\x00e\x00p\x00o\x00r\x00t\x00i\x00n\x00g\x00\\"
        b"\x00c\x00o\x00n\x00s\x00e\x00n\x00t\x00\x00"
    )
    policy_regkey = (
        b"\x00D\x00e\x00f\x00a\x00u\x00l\x00t\x00C\x00o\x00n"
        b"\x00s\x00e\x00n\x00t\x00\x00"
    )
    test = win_lgpo._regexSearchKeyValueCombo(
        policy_data=policy_data,
        policy_regpath=policy_regpath,
        policy_regkey=policy_regkey,
    )
    assert test == policy_data


def test__encode_string():
    """
    ``_encode_string`` should return a null terminated ``utf-16-le`` encoded
    string when a string value is passed
    """
    encoded_null = chr(0).encode("utf-16-le")
    encoded_value = b"".join(["Salt is awesome".encode("utf-16-le"), encoded_null])
    value = win_lgpo._encode_string("Salt is awesome")
    assert value == encoded_value


def test__encode_string_empty_string():
    """
    ``_encode_string`` should return an encoded null when an empty string
    value is passed
    """
    value = win_lgpo._encode_string("")
    encoded_null = chr(0).encode("utf-16-le")
    assert value == encoded_null


def test__encode_string_error():
    """
    ``_encode_string`` should raise an error if a non-string value is passed
    """
    pytest.raises(TypeError, win_lgpo._encode_string, [1])
    test_list = ["item1", "item2"]
    pytest.raises(TypeError, win_lgpo._encode_string, [test_list])
    test_dict = {"key1": "value1", "key2": "value2"}
    pytest.raises(TypeError, win_lgpo._encode_string, [test_dict])


def test__encode_string_none():
    """
    ``_encode_string`` should return an encoded null when ``None`` is passed
    """
    value = win_lgpo._encode_string(None)
    encoded_null = chr(0).encode("utf-16-le")
    assert value == encoded_null


def test__multi_string_get_transform_list():
    """
    ``_multi_string_get_transform`` should return the list when a list is
    passed
    """
    test_value = ["Spongebob", "Squarepants"]
    value = win_lgpo._policy_info._multi_string_get_transform(item=test_value)
    assert value == test_value


def test__multi_string_get_transform_none():
    """
    ``_multi_string_get_transform`` should return "Not Defined" when
    ``None`` is passed
    """
    test_value = None
    value = win_lgpo._policy_info._multi_string_get_transform(item=test_value)
    assert value == "Not Defined"


def test__multi_string_get_transform_invalid():
    """
    ``_multi_string_get_transform`` should return "Not Defined" when
    ``None`` is passed
    """
    test_value = "Some String"
    value = win_lgpo._policy_info._multi_string_get_transform(item=test_value)
    assert value == "Invalid Value"


def test__multi_string_put_transform_list():
    """
    ``_multi_string_put_transform`` should return the list when a list is
    passed
    """
    test_value = ["Spongebob", "Squarepants"]
    value = win_lgpo._policy_info._multi_string_put_transform(item=test_value)
    assert value == test_value


def test__multi_string_put_transform_none():
    """
    ``_multi_string_put_transform`` should return ``None`` when
    "Not Defined" is passed
    """
    test_value = "Not Defined"
    value = win_lgpo._policy_info._multi_string_put_transform(item=test_value)
    assert value is None


def test__multi_string_put_transform_list_from_string():
    """
    ``_multi_string_put_transform`` should return a list when a comma
    delimited string is passed
    """
    test_value = "Spongebob,Squarepants"
    value = win_lgpo._policy_info._multi_string_put_transform(item=test_value)
    assert value == ["Spongebob", "Squarepants"]


def test__multi_string_put_transform_invalid():
    """
    ``_multi_string_put_transform`` should return "Invalid" value if neither
    string nor list is passed
    """
    test_value = None
    value = win_lgpo._policy_info._multi_string_put_transform(item=test_value)
    assert value == "Invalid Value"
